/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 1002 $
 * $Date: 2020-04-10 23:31:24 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.editor2;

import javax.swing.table.AbstractTableModel;

import at.lcm.bu21.general.dtypes.editor2.TDataModel.TStringModel;

/**
 * This is a table model for a JTable to view TStrings. It based on
 * AbstractTableModel.
 * 
 * @author Johannes Kaisinger
 */
public class TStringTableModel extends AbstractTableModel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/**
	 * Local reference of the tStringModel. It allows to manipulate the TString.
	 */
	private TStringModel tStringModel;

	/**
	 * Creates an object
	 * 
	 * @param tStringModel
	 *            The TStringModel
	 */
	public TStringTableModel(TStringModel tStringModel) {
		this.tStringModel = tStringModel;
	}

	/**
	 * Get Column name
	 * 
	 * @param column
	 *            Index of column.
	 */
	@Override
	public String getColumnName(int column) {
		return "" + 0;
	}

	/**
	 * Get row count of tStringModel.
	 * 
	 * @return rowCount
	 */
	@Override
	public int getRowCount() {
		return 1;
	}

	/**
	 * Get column count of tStringModel.
	 * 
	 * @return columnCount
	 */
	@Override
	public int getColumnCount() {
		return 1;
	}

	/**
	 * Get column class.
	 * 
	 * @return Class
	 */
	@Override
	public Class<String> getColumnClass(int columnIndex) {
		return String.class;
	}

	/**
	 * Get if a cell is editable.
	 * 
	 * @param row
	 *            Index of row.
	 * @param column
	 *            Index of column.
	 * @return editable
	 */
	@Override
	public boolean isCellEditable(int row, int column) {
		return true;
	}

	/**
	 * Get a cell value.
	 * 
	 * @param row
	 *            Index of row.
	 * @param column
	 *            Index of column.
	 * @return value Value of cell (String).
	 */
	@Override
	public String getValueAt(int row, int column) {
		return this.tStringModel.getString();

	}

	/**
	 * Set a cell value.
	 * 
	 * @param value
	 *            New value of the cell.
	 * @param row
	 *            Index of row.
	 * @param column
	 *            Index of column.
	 */
	@Override
	public void setValueAt(Object value, int row, int column) {
		this.tStringModel.setString(value.toString());
		this.fireTableCellUpdated(row, column);
	}
}
