/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 999 $
 * $Date: 2020-04-10 22:54:35 +0200 (Fri, 10 Apr 2020) $
 */
package at.lcm.bu21.general.dtypes.unit;

/**
 * Prefixes for units
 *
 * @author Werner Koppelsttter
 */
public enum Prefix {

	/** yotta */
	YOTTA("Y", 1e24),
	/** zetta */
	ZETTA("Z", 1e21),
	/** exa */
	EXA("E", 1e18),
	/** peta */
	PETA("P", 1e15),
	/** tera */
	TERA("T", 1e12),
	/** giga */
	GIGA("G", 1e9),
	/** mega */
	MEGA("M", 1e6),
	/** kilo */
	KILO("k", 1e3),
	/** hecto */
	HECTO("h", 1e2),
	/** deca */
	DECA("da", 1e1),
	/** deci */
	DECI("d", 1e-1),
	/** centi */
	CENTI("c", 1e-2),
	/** milli */
	MILLI("m", 1e-3),
	/** micro */
	MICRO("", 1e-6),
	/** MICRO */
	MICOR2("u", 1e-6),
	/** nano */
	NANO("n", 1e-9),
	/** pico */
	PICO("p", 1e-12),
	/** femto */
	FEMTO("f", 1e-15),
	/** atto */
	ATTO("a", 1e-18),
	/** zepto */
	ZEPTO("z", 1e-21),
	/** yocto */
	YOCTO("y", 1e-24);

	/** Symbol of prefix */
	private String symbol;

	/** Gain of prefix */
	private double gain;

	/**
	 * Define prefix
	 * 
	 * @param symbol
	 *            Symbol
	 * @param gain
	 *            Gain
	 */
	private Prefix(String symbol, double gain) {
		this.symbol = symbol;
		this.gain = gain;
	}

	/**
	 * Get prefix symbol
	 * 
	 * @return Symbol
	 */
	public String getSymbol() {
		return this.symbol;
	}

	/**
	 * Get prefix gain
	 * 
	 * @return Gain
	 */
	public double getGain() {
		return this.gain;
	}

}
