# -*- coding: utf-8 -*-

from dtypes.tperiodic import TPeriodic
from dtypes.tarray import TArray
import copy
import numpy as np

from math import pi

class TPeriodicMatrix(TArray):
            
    _tag="PeriodicMatrix" 
         
    def __new__(cls, nrows, ncols):
        shape=[nrows, ncols]
        obj=np.full(shape, TPeriodic(), dtype=TPeriodic)
        return obj.view(cls)         
         
    def __setitem__(self, tup, val):
        if len(tup) != 2:
            raise Exception("index requires 2 dimensions!")
        super(TPeriodicMatrix, self).__setitem__(tup, TPeriodic(val))
        
    def __getitem__(self, tup):
        if len(tup) != 2:
            raise Exception("index requires 2 dimensions!")
        return super(TPeriodicMatrix, self).__getitem__(tup)
        
    def __add__(self, other):
        return self._binop(lambda x,y: x+y, other)

    def scale(self, sc):
        return self._unop(lambda x: x*sc)

    def _unop(self, fnc):
        """
        evaluate unary operator result=fnc(self) on each element
        """
        theClone=copy.deepcopy(self)        

        for row in range(0, self.shape[0]):
            for col in range(0, self.shape[1]):
                theClone[row, col] = fnc(self[row, col])
            
        return theClone
        
    def _binop(self, fnc, other):
        """
        evaluate binary operator result=fnc(self, other) on each element
        """
        theClone=copy.deepcopy(self)        
        
        for row in range(0, self.shape[0]):
            for col in range(0, self.shape[1]):
                theClone[row, col] = fnc(self[row, col], other[row, col])
            
        return theClone
        
        
    def showPreview(self):
        """
        Show simple preview of the signal
        """        
        import matplotlib.pyplot as plt
        from matplotlib.ticker import FixedLocator, FormatStrFormatter
                        
                
        f, axes = plt.subplots(self.shape[0], self.shape[1], sharex='col', sharey='row')
        for rowidx, row in enumerate(axes):
            for colidx, a in enumerate(row):
                
                tp=self[rowidx, colidx]
                phi=tp.getPhiVec()
                val=tp.valueAt(phi)
                a.plot(phi*180./pi, val)
                a.grid(True)
                
                a.xaxis.grid(True, 'minor')
                a.xaxis.set_major_locator(FixedLocator(np.arange(0,360,90)))
                a.xaxis.set_major_formatter(FormatStrFormatter(u'%d°'))
                a.xaxis.set_minor_locator(FixedLocator(np.arange(0,360,30)))
                
        plt.savefig('f:\\test.png')