#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 1024 $
$LastChangedDate:: 2020-06-25 18:45:41 +0200#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import numpy as np
import lcm.utils.pytperiodic as pm

try:
    import jep
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.dtypes import TNull
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.dtypes import TBoolean
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.dtypes import TString
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.dtypes import TNumeric
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.dtypes import TNumeric
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.dtypes import TPeriodic
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.dtypes import TPeriodicMatrix
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.dtypes import TArray
    # noinspection PyUnresolvedReferences
    from at.lcm.bu21.general.dtypes import TStruct
    # noinspection PyUnresolvedReferences
    from org.apache.commons.math3.linear import BlockRealMatrix
except ImportError as iex:
    pass
except Exception as ex:
    pass


class TDataTesting:

    @staticmethod
    def getPythonNone():
        return None

    @staticmethod
    def getTNull():
        return TNull()

    @staticmethod
    def getPythonBool(boolean=True):
        """

        Parameters
        ----------
        boolean : bool

        Returns
        -------
        bool
        """
        return boolean

    @staticmethod
    def getTBoolean(boolean=True):
        """

        Parameters
        ----------
        boolean : bool

        Returns
        -------
        TBoolean
        """
        return TBoolean(boolean)

    @staticmethod
    def getPythonString():
        return 'str'

    @staticmethod
    def getTString():
        return TString('str')

    @staticmethod
    def getComplexNumpyVector(size=3):
        return np.arange(size) - 1j * np.arange(size)

    # noinspection PyUnresolvedReferences
    @staticmethod
    def getComplexTNumericVector(size=3):

        j_real_array = jep.jarray(size, jep.JDOUBLE_ID, 0)
        j_imag_array = jep.jarray(size, jep.JDOUBLE_ID, 0)
        for index in range(size):
            j_real_array[index] = float(index)
            j_imag_array[index] = float(-index)
        return TNumeric(j_real_array, j_imag_array)

    @staticmethod
    def getNumpyMatrix(size1=3, size2=3):
        return np.ones((size1, size2)) - 1j * np.ones((size1, size2))

    @staticmethod
    def getTNumericMatrix(size1=3, size2=3):
        j_real_matrix = BlockRealMatrix(size1, size2)
        j_imag_matrix = BlockRealMatrix(size1, size2)
        for row in range(3):
            for col in range(3):
                j_real_matrix.setEntry(row, col, 1)
                j_imag_matrix.setEntry(row, col, -1)
        return TNumeric(j_real_matrix, j_imag_matrix)

    @classmethod
    def getTPeriodicMatrix(cls, nrows, ncols, order, magnitude, angle):
        jobj = TPeriodicMatrix(nrows, ncols)
        for r in range(nrows):
            for c in range(ncols):
                # jobj.set(r, c, TPeriodic(int(r * order), float(c * magnitude), float(angle)))
                jobj.set(r, c, cls.getTPeriodic(r * order + 1, c * magnitude, angle))
        return jobj

    @classmethod
    def getPyTPeriodicMatrix(cls, nrows, ncols, order, magnitude, angle) -> pm.PyTPeriodicMatrix:
        xc = np.zeros((nrows, ncols), dtype=np.object)
        for r in range(nrows):
            for c in range(ncols):
                # xc[r, c] = pm.PyTPeriodicMatrix(r * order + 1, c * magnitude, angle)
                xc[r, c] = cls.getPyTPeriodic(r * order + 1, c * magnitude, angle)

        pyobj = pm.PyTPeriodicMatrix(xc, mode='objarray')
        return pyobj

    @staticmethod
    def getTPeriodic(order, magnitude, angle):

        jobj = TPeriodic(int(order), float(magnitude), float(angle))
        # jobj = TPeriodic.getRandom()
        return jobj

    @staticmethod
    def getPyTPeriodic(order, magnitude, angle) -> pm.PyTPeriodicMatrix:
        pyobj = pm.PyTPeriodicMatrix(1, magnitude, angle)
        return pyobj

    @classmethod
    def getPythonNumpyObjectArray(cls):
        tarray = np.array([], dtype=np.dtype(object))
        tnull = cls.getPythonNone()
        tboolean = cls.getPythonBool()
        tstring = cls.getPythonString()
        tnumeric = cls.getComplexNumpyVector()

        for tdata in [tnull, tboolean, tstring, tnumeric]:
            tarray = np.append(tarray, tdata)
        return tarray

    @classmethod
    def getTArray(cls):

        tarray = TArray()
        tnull = cls.getTNull()
        tboolean = cls.getTBoolean()
        tstring = cls.getTString()
        tnumeric = cls.getComplexTNumericVector()

        index = 0
        for tdata in [tnull, tboolean, tstring, tnumeric]:
            tarray.set(0, index, tdata)
            index += 1
        return tarray

    @classmethod
    def getDict(cls) -> dict:
        tnull = cls.getPythonNone()
        tboolean = cls.getPythonBool()
        tstring = cls.getPythonString()
        tnumeric = cls.getComplexNumpyVector()

        d = dict(TNull=tnull, TBoolean=tboolean, TString=tstring, TNumeric=tnumeric)
        return d

    @classmethod
    def getTStruct(cls):
        tstruct = TStruct()
        tnull = cls.getTNull()
        tboolean = cls.getTBoolean()
        tstring = cls.getTString()
        tnumeric = cls.getComplexTNumericVector()

        tstruct.put('TNull', tnull)
        tstruct.put('TBoolean', tboolean)
        tstruct.put('TString', tstring)
        tstruct.put('TNumeric', tnumeric)

        return tstruct
