/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9715 $
 * $Date: 2020-03-19 10:01:15 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.symspace.optimizer.views;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import at.lcm.bu21.optim.core.project.ComputeProject;
import at.lcm.bu21.optim.gui.project.PanelPlots;
import at.lcm.symspace.gui.views.ToolBar;
import at.lcm.symspace.optimizer.OptimizerController;

public class OptimizerStatusBar extends ToolBar {
	final OptimizerController controller;

	/** Date formatter */
	private SimpleDateFormat sdf = new SimpleDateFormat("H:mm d-MMMMM-YYYY");

	private JTextField txtSavePoints;
	private JTextField txtSolved;
	private JTextField txtBuffer;
	private JComboBox<String> plotType;
	private JSpinner numberParetos;
	private JTextField txtStatus;
	private JCheckBox plotHypervolumne;
	private JCheckBox plotMapping;
	private JTextField txtStopContitionSavepoint;
	private JTextField txtStopContitionTime;
	private JProgressBar progressBar;

	public OptimizerStatusBar(OptimizerController controller) {
		JLabel label;
		Dimension dim;
		String str;

		this.controller = controller;

		label = new JLabel("SP:");
		label.setToolTipText("Savepoints");
		this.add(label);
		this.txtSavePoints = new JTextField(5);
		this.add(this.txtSavePoints);
		this.txtSavePoints.setMaximumSize(this.txtSavePoints.getPreferredSize());
		this.txtSavePoints.setEnabled(false);

		this.addSeparator();

		label = new JLabel("SI:");
		label.setToolTipText("Solved inividuals");
		this.add(label);
		this.txtSolved = new JTextField(5);
		this.add(this.txtSolved);
		this.txtSolved.setMaximumSize(this.txtSolved.getPreferredSize());
		this.txtSolved.setEnabled(false);

		this.addSeparator();

		label = new JLabel("Buf:");
		label.setToolTipText("Buffer size");
		this.add(label);
		this.txtBuffer = new JTextField(5);
		this.add(this.txtBuffer);
		this.txtBuffer.setMaximumSize(this.txtBuffer.getPreferredSize());
		this.txtBuffer.setEnabled(false);

		this.addSeparator();

		label = new JLabel("Plot:");
		label.setToolTipText("Type of plot");
		this.add(label);
		this.plotType = new JComboBox<String>(PanelPlots.listPlotTypes());
		this.add(this.plotType);
		this.plotType.setMaximumSize(this.plotType.getMinimumSize());
		this.plotType.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				// FIXME controller
				if (OptimizerStatusBar.this.controller.getViewManager() != null) {
					OptimizerStatusBar.this.controller.getViewManager().plots
							.setPlotType((String) OptimizerStatusBar.this.plotType.getSelectedItem());
					OptimizerStatusBar.this.refresh();
				}
			}
		});

		this.addSeparator();

		label = new JLabel("Display # SP:");
		label.setToolTipText("Number of displayed pareto-fronts");
		this.add(label);
		// FIXME
		this.numberParetos = new JSpinner();
		this.add(this.numberParetos);
		dim = this.numberParetos.getPreferredSize();
		dim.width *= 2;
		this.numberParetos.setMaximumSize(dim);
		this.numberParetos.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent event) {
				// FIXME controller
				if (OptimizerStatusBar.this.controller.getViewManager() != null) {
					OptimizerStatusBar.this.controller.getViewManager().plots
							.setNumPlotSavepoints((Integer) OptimizerStatusBar.this.numberParetos.getValue());
				}
			}
		});

		this.addSeparator();

		label = new JLabel("Status:");
		label.setToolTipText("Status of optimization");
		this.add(label);
		this.txtStatus = new JTextField(10);
		this.txtStatus.setForeground(Color.RED);
		this.add(this.txtStatus);
		this.txtStatus.setMaximumSize(this.txtStatus.getPreferredSize());
		this.txtStatus.setEnabled(false);

		this.addSeparator();

		this.plotHypervolumne = new JCheckBox("HV");
		this.plotHypervolumne.setToolTipText("plot hypervolumne");
		this.add(this.plotHypervolumne);
		this.plotHypervolumne.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				// FIXME controller
				if (OptimizerStatusBar.this.controller.getViewManager() != null) {
					OptimizerStatusBar.this.controller.getViewManager().plots
							.setPlotHypervolume(OptimizerStatusBar.this.plotHypervolumne.isSelected());
				}
			}
		});

		this.addSeparator();

		this.plotMapping = new JCheckBox("MM");
		this.plotMapping.setToolTipText("Mark mapping");
		this.add(this.plotMapping);
		this.plotMapping.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				// FIXME controller
				if (OptimizerStatusBar.this.controller.getViewManager() != null) {
					OptimizerStatusBar.this.controller.getViewManager().plots
							.setMarkMapping(OptimizerStatusBar.this.plotMapping.isSelected());
				}
			}
		});

		this.addSeparator();

		label = new JLabel("Stop@SP:");
		label.setToolTipText("Stop optimization at Savepoint");
		this.add(label);
		this.txtStopContitionSavepoint = new JTextField(10);
		this.txtStopContitionSavepoint.setForeground(Color.RED);
		this.add(this.txtStopContitionSavepoint);
		this.txtStopContitionSavepoint.setMaximumSize(this.txtStopContitionSavepoint.getPreferredSize());
		this.txtStopContitionSavepoint.setEnabled(false);
		this.txtStopContitionSavepoint.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				System.out.println("SP: " + e.getClickCount());
				if (e.getClickCount() > 1) {
					OptimizerStatusBar.this.controller.updateStopContition();
				}
			}
		});

		this.addSeparator();

		label = new JLabel("Stop@T:");
		label.setToolTipText("Stop optimization at time");
		this.add(label);
		this.txtStopContitionTime = new JTextField(15);
		this.txtStopContitionTime.setForeground(Color.RED);
		this.add(this.txtStopContitionTime);
		this.txtStopContitionTime.setMaximumSize(this.txtStopContitionTime.getPreferredSize());
		this.txtStopContitionTime.setEnabled(false);
		this.txtStopContitionTime.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				System.out.println("Time: " + e.getClickCount());
				if (e.getClickCount() > 1) {
					OptimizerStatusBar.this.controller.updateStopContition();
				}
			}
		});

		this.addSeparator();

		// define progress-bar
		this.progressBar = new JProgressBar();
		this.progressBar.setString("Status");
		this.progressBar.setMinimum(0);
		this.progressBar.setIndeterminate(true);
		this.progressBar.setStringPainted(true);
		this.progressBar.setVisible(false);
		this.add(this.progressBar);

	}

	public void showProgressBar(final String init, final int max) {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				if (max > 0) {
					OptimizerStatusBar.this.progressBar.setMaximum(max);
					OptimizerStatusBar.this.progressBar.setIndeterminate(false);
				} else {
					OptimizerStatusBar.this.progressBar.setIndeterminate(true);
				}
				OptimizerStatusBar.this.progressBar.setVisible(true);
				OptimizerStatusBar.this.progressBar.setString(init);
			}
		});
	}

	public void updateProgressBar(final int val, final String status) {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				OptimizerStatusBar.this.progressBar.setValue(val);
				OptimizerStatusBar.this.progressBar.setString(status);
			}
		});

	}

	public void hideProgressBar() {
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				OptimizerStatusBar.this.progressBar.setVisible(false);
			}
		});

	}

	@Override
	public void refresh() {
		int num;
		ComputeProject proj = null;
		OptimizerViewManager view;

		if ((this.controller != null) && (this.controller.getOptimization() != null)) {
			proj = this.controller.getOptimization().getProjectControl();
			view = this.controller.getViewManager();
			// Update # of Generations
			num = proj.getProblem().getNumSavePoints();
			if (proj.getProblem().existTempSavePoint()) {
				num--;
			}

			this.txtSavePoints.setText(Integer.toString(num));
			this.txtSolved.setText(Integer.toString(proj.getNumberSolved()));
			this.txtBuffer.setText(Integer.toString(proj.getBufferSize()));
			this.plotType.setEnabled(true);
			this.plotType.setSelectedItem(view.plots.getPlotType());
			this.numberParetos.setEnabled(true);
			this.numberParetos.setValue(view.plots.getNumPlotSavepoints());
			this.plotHypervolumne.setEnabled(true);
			this.plotHypervolumne.setSelected(view.plots.getPlotHypervolume());
			this.txtStatus.setText(proj.getStatus());
			this.plotMapping.setEnabled(true);
			this.plotMapping.setSelected(view.plots.getMarkMapping());
			if (proj.getStopConditionSavepoint() != Integer.MAX_VALUE) {
				this.txtStopContitionSavepoint.setText(Integer.toString(proj.getStopConditionSavepoint()));
			} else {
				this.txtStopContitionSavepoint.setText("");
			}
			if (proj.getStopConditionTime() != Long.MAX_VALUE) {
				this.txtStopContitionTime.setText(this.sdf.format(new Date(proj.getStopConditionTime())));
			} else {
				this.txtStopContitionTime.setText("");
			}
		} else {
			this.txtSavePoints.setText("");
			this.txtSolved.setText("");
			this.txtBuffer.setText("");
			this.plotType.setEnabled(false);
			this.numberParetos.setEnabled(false);
			this.txtStatus.setText("");
			this.plotHypervolumne.setEnabled(false);
			this.plotMapping.setEnabled(false);
			this.txtStopContitionSavepoint.setText("");
			this.txtStopContitionTime.setText("");
		}
	}
}
