/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.docu;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import at.lcm.bu21.general.utils.Clone;

/**
 * Dialog for set report-structure
 * 
 * @author Werner Koppelsttter
 */
public class ReportStructureDialog {

	/** dialog */
	JDialog dialog;
	/** CheckBox for summary */
	JCheckBox cbSummary = new JCheckBox("Summary");
	/** CheckBox for Detail */
	JCheckBox cbDetail = new JCheckBox("Detail");
	/** CheckBox for template */
	JCheckBox cbTemplate = new JCheckBox("Template");
	/** CheckBox for parameter */
	JCheckBox cbParameter = new JCheckBox("Parameter");
	/** CheckBox for info */
	JCheckBox cbInfo = new JCheckBox("Info");
	/** CheckBox for import/export */
	JCheckBox cbImExport = new JCheckBox("Import/Export");
	/** CheckBox for used/set */
	JCheckBox cbUsedSet = new JCheckBox("Used/Set");
	/** CheckBox for files */
	JCheckBox cbFiles = new JCheckBox("Files");
	/** CheckBox for function */
	JCheckBox cbFunction = new JCheckBox("Function");
	/** CheckBox for package */
	JCheckBox cbPackage = new JCheckBox("Package");
	/** CheckBox for data-sheet */
	JCheckBox cbDatasheet = new JCheckBox("Datasheet");

	/**
	 * Structure of report
	 */
	ReportStructure structure;

	/**
	 * Open dialog with default structure
	 * 
	 * @param parent
	 *            Parent frame
	 * @return Structure of report
	 */
	public ReportStructure show(Frame parent) {
		return this.show(parent, new ReportStructure());
	}

	/**
	 * Open dialog with given structure
	 * 
	 * @param parent
	 *            Parent frame
	 * @param structure
	 *            Pattern structure
	 * @return Structure of report
	 */
	public ReportStructure show(Frame parent, ReportStructure structure) {

		try {
			this.structure = (ReportStructure) Clone.deepClone(structure);
		} catch (Exception e) {
		}
		this.refresh();
		this.dialog = this.init(parent);
		this.dialog.setLocationRelativeTo(parent);
		this.dialog.setVisible(true);

		return this.structure;
	}

	/**
	 * Initialize dialog
	 * 
	 * @param parent
	 *            Parent frame
	 * @return Dialog
	 */
	protected JDialog init(Frame parent) {
		JDialog dialog;
		JPanel mainPanel, panel, item;
		JButton button;
		JLabel label;
		Font font;

		dialog = new JDialog(parent, "Select report structure", true);
		dialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				ReportStructureDialog.this.structure = null;
				ReportStructureDialog.this.dialog.setVisible(false);
				ReportStructureDialog.this.dialog.dispose();
			}
		});
		dialog.setLayout(new BorderLayout());

		mainPanel = new JPanel(new BorderLayout());
		dialog.add(mainPanel, BorderLayout.CENTER);
		dialog.add(new JPanel(), BorderLayout.WEST);
		dialog.add(new JPanel(), BorderLayout.EAST);
		dialog.add(new JPanel(), BorderLayout.NORTH);
		dialog.add(new JPanel(), BorderLayout.SOUTH);

		// top area
		label = new JLabel("Structure of report");
		font = label.getFont();
		font = font.deriveFont(Font.BOLD, font.getSize() * 1.4f);
		label.setFont(font);
		mainPanel.add(label, BorderLayout.NORTH);

		//
		panel = new JPanel(new GridLayout(11, 1));
		mainPanel.add(panel, BorderLayout.CENTER);

		// summary
		this.cbSummary.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setSummary(ReportStructureDialog.this.cbSummary
								.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(this.cbSummary);
		panel.add(item);

		// detail
		this.cbDetail.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setDetail(ReportStructureDialog.this.cbDetail
								.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(this.cbDetail);
		panel.add(item);

		// template
		this.cbTemplate.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setTemplate(ReportStructureDialog.this.cbTemplate
								.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(new JPanel());
		item.add(this.cbTemplate);
		panel.add(item);

		// parameter
		this.cbParameter.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setParameter(ReportStructureDialog.this.cbParameter
								.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(new JPanel());
		item.add(this.cbParameter);
		panel.add(item);

		// info
		this.cbInfo.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setInfo(ReportStructureDialog.this.cbInfo.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(new JPanel());
		item.add(new JPanel());
		item.add(this.cbInfo);
		panel.add(item);

		// import/export
		this.cbImExport.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setImExport(ReportStructureDialog.this.cbImExport
								.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(new JPanel());
		item.add(new JPanel());
		item.add(this.cbImExport);
		panel.add(item);

		// used/set
		this.cbUsedSet.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setUsedSet(ReportStructureDialog.this.cbUsedSet
								.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(new JPanel());
		item.add(new JPanel());
		item.add(this.cbUsedSet);
		panel.add(item);

		// files
		this.cbFiles.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setFiles(ReportStructureDialog.this.cbFiles
								.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(new JPanel());
		item.add(new JPanel());
		item.add(this.cbFiles);
		panel.add(item);

		// function
		this.cbFunction.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setFunction(ReportStructureDialog.this.cbFunction
								.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(this.cbFunction);
		panel.add(item);

		// package
		this.cbPackage.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setPackage(ReportStructureDialog.this.cbPackage
								.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(this.cbPackage);
		panel.add(item);

		// data-sheet
		this.cbDatasheet.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ReportStructureDialog.this.structure
						.setDatasheet(ReportStructureDialog.this.cbDatasheet
								.isSelected());
				ReportStructureDialog.this.refresh();
			}
		});
		item = new JPanel(new FlowLayout(FlowLayout.LEFT));
		item.add(this.cbDatasheet);
		panel.add(item);

		// bottom area
		panel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		mainPanel.add(panel, BorderLayout.SOUTH);

		button = new JButton("OK");
		button.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				ReportStructureDialog.this.dialog.setVisible(false);
				ReportStructureDialog.this.dialog.dispose();
			}
		});
		panel.add(button, BorderLayout.WEST);

		button = new JButton("Cancel");
		button.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				ReportStructureDialog.this.structure = null;
				ReportStructureDialog.this.dialog.setVisible(false);
				ReportStructureDialog.this.dialog.dispose();
			}
		});
		panel.add(button, BorderLayout.EAST);

		dialog.pack();

		return dialog;
	}

	/**
	 * Refresh drawing with structure
	 */
	protected void refresh() {
		this.cbSummary.setSelected(this.structure.getSummary());
		this.cbDetail.setSelected(this.structure.getDetail());
		this.cbTemplate.setSelected(this.structure.getTemplate());
		this.cbParameter.setSelected(this.structure.getParameter());
		this.cbInfo.setSelected(this.structure.getInfo());
		this.cbImExport.setSelected(this.structure.getImExport());
		this.cbUsedSet.setSelected(this.structure.getUsedSet());
		this.cbFiles.setSelected(this.structure.getFiles());
		this.cbFunction.setSelected(this.structure.getFunction());
		this.cbPackage.setSelected(this.structure.getPackage());
		this.cbDatasheet.setSelected(this.structure.getDatasheet());
	}

}
