/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure;

import java.rmi.RemoteException;

import at.lcm.bu21.general.dtypes.TData;

/**
 * Interface of function
 * 
 * @author Werner Koppelsttter
 */
public interface IParamFunction extends IParamContainer, IFunction {

	/**
	 * Get identifier of function
	 * 
	 * @return ID
	 * @throws RemoteException
	 */
	public String getFunctionID() throws RemoteException;

	/**
	 * Get name of class
	 * 
	 * @return Name
	 * @throws RemoteException
	 */
	public String getClassName() throws RemoteException;

	/**
	 * Get path of class
	 * 
	 * @return Path
	 * @throws RemoteException
	 */
	public String getClassPath() throws RemoteException;

	/**
	 * Has function-object a preview method
	 * 
	 * @return Has preview
	 * @throws RemoteException
	 */
	public boolean hasPreview() throws RemoteException;

	/**
	 * Has function-object a show method
	 * 
	 * @return Has show
	 * @throws RemoteException
	 */
	public boolean hasShow() throws RemoteException;

	/**
	 * Open function model
	 * 
	 * @throws RemoteException
	 */
	public void show() throws RemoteException;

	/**
	 * Get value of import
	 * 
	 * @param name
	 *            Name of import
	 * @return Value of import
	 * @throws RemoteException
	 * @throws Exception
	 */
	public TData getImportValue(String name) throws RemoteException, Exception;

	/**
	 * Get value of export
	 * 
	 * @param name
	 *            Name of export
	 * @return Value of export
	 * @throws RemoteException
	 * @throws Exception
	 */
	public TData getExportValue(String name) throws RemoteException, Exception;

	/**
	 * Set value of export
	 * 
	 * @param name
	 *            Name of export
	 * @param data
	 *            Value for export
	 * @throws RemoteException
	 * @throws Exception
	 */
	public void setExportValue(String name, TData data) throws RemoteException, Exception;

	/**
	 * Get unit of import
	 * 
	 * @param name
	 *            Name of import
	 * @return Unit of import
	 * @throws RemoteException
	 */
	public String getImportUnit(String name) throws RemoteException;

	/**
	 * Get unit of export
	 * 
	 * @param name
	 *            Name of export
	 * @return Unit of export
	 * @throws RemoteException
	 */
	public String getExportUnit(String name) throws RemoteException;
}
