/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.client;

import java.rmi.RemoteException;

import at.lcm.bu21.magopt.structure.IDependItem;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.PortDesc;

/**
 * Client adapter for depend-item interface
 * 
 * @author Werner Koppelsttter
 */
public class CDependItem implements IDependItem {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Depend-item class */
	private IDependItem item;

	/**
	 * Create client adapter for depend-item interface
	 * 
	 * @param item
	 *            Depend item
	 */
	public CDependItem(IDependItem item) {
		this.item = item;
	}

	@Override
	public String getVarName() throws RemoteException {
		return this.item.getVarName();
	}

	@Override
	public PortDesc getVarDesc() throws RemoteException {
		return this.item.getVarDesc();
	}

	@Override
	public String getParam() throws RemoteException {
		return this.item.getParam();
	}

	@Override
	public IParameter getHandle() throws RemoteException {
		IParameter param;
		IParameter clientParam = null;

		param = this.item.getHandle();
		if (param != null) {
			clientParam = param.getClientImpl();
		}

		return clientParam;
	}

	@Override
	public CDependItem getClientImpl() {
		return this;
	}
}
