/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.client;

import java.rmi.RemoteException;

import at.lcm.bu21.magopt.structure.IParamComboBox;

/**
 * Client adapter for combo-box interface
 * 
 * @author Werner Koppelsttter
 */
public class CParamComboBox extends CParamField implements IParamComboBox {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Combo-box class */
	private IParamComboBox param;

	/**
	 * Create client adapter for combo-box interface
	 * 
	 * @param param
	 *            Combo box
	 */
	public CParamComboBox(IParamComboBox param) {
		super(param);
		this.param = param;
	}

	@Override
	public String[] getList() throws RemoteException {
		return this.param.getList();
	}

	@Override
	public int getComboIndex() throws RemoteException {
		return this.param.getComboIndex();
	}

	@Override
	public void setComboIndex(int comboIndex) throws Exception {
		this.param.setComboIndex(comboIndex);
	}

	@Override
	public String getComboList() throws RemoteException {
		return this.param.getComboList();
	}

	@Override
	public void setComboList(String comboList) throws Exception {
		this.param.setComboList(comboList);
	}

	@Override
	public CParamComboBox getClientImpl() {
		return this;
	}
}
