/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.client;

import java.rmi.RemoteException;
import java.util.Map;

import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.magopt.structure.IDepend;
import at.lcm.bu21.magopt.structure.IParamFunction;

/**
 * Client adapter for function interface
 * 
 * @author Werner Koppelsttter
 */
public abstract class CParamFunction extends CParamContainer implements IParamFunction {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Function class */
	private IParamFunction param;

	/**
	 * Create client adapter for function interface
	 * 
	 * @param param
	 *            Function
	 */
	public CParamFunction(IParamFunction param) {
		super(param);
		this.param = param;
	}

	@Override
	public String getFunctionID() throws RemoteException {
		return this.param.getFunctionID();
	}

	@Override
	public boolean getUpdateRequired() throws RemoteException {
		return this.param.getUpdateRequired();
	}

	@Override
	public boolean getUpdateRequired(int index) throws RemoteException {
		return this.param.getUpdateRequired(index);
	}

	@Override
	public void setUpdateRequired(boolean update) throws RemoteException {
		this.param.setUpdateRequired(update);
	}

	@Override
	public void setUpdateRequired(int index, boolean update) throws RemoteException {
		this.param.setUpdateRequired(index, update);
	}

	@Override
	public IDepend getDepend() throws RemoteException {
		return this.param.getDepend().getClientImpl();
	}

	@Override
	public String getClassName() throws RemoteException {
		return this.param.getClassName();
	}

	@Override
	public String getClassPath() throws RemoteException {
		return this.param.getClassPath();
	}

	@Override
	public boolean hasPreview() throws RemoteException {
		return this.param.hasPreview();
	}

	@Override
	public boolean hasShow() throws RemoteException {
		return this.param.hasShow();
	}

	@Override
	public void show() throws RemoteException {
		this.param.show();
	}

	@Override
	public int update(Map<String, Object> args) throws Exception, RemoteException {
		return this.param.update(args);
	}

	@Override
	public void refreshDepend(boolean dirty, boolean load) throws RemoteException {
		this.param.refreshDepend(dirty, load);
	}

	@Override
	public TData getImportValue(String name) throws Exception {
		return this.param.getImportValue(name);
	}

	@Override
	public TData getExportValue(String name) throws Exception {
		return this.param.getExportValue(name);
	}

	@Override
	public void setExportValue(String name, TData data) throws RemoteException, Exception {
		this.param.setExportValue(name, data);
	}

	@Override
	public String getImportUnit(String name) throws RemoteException {
		return this.param.getImportUnit(name);
	}

	@Override
	public String getExportUnit(String name) throws RemoteException {
		return this.param.getExportUnit(name);
	}
}
