/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.converter;

import at.lcm.bu21.general.utils.XMLNode;

/**
 * Convert to V_0_13.
 * 
 * - Remove lock flag from Patameter <br>
 * - Add lock flag to ParamField
 * 
 * @author Werner Koppelsttter
 */
class Converter_13 {

	/**
	 * Convert the given node
	 * 
	 * @param node
	 *            Data to convert
	 * @return Changed data node
	 */
	static XMLNode convert(XMLNode node) {
		// Convert Parameter
		Converter_13.removeLock(node);
		Converter_13.addLock(node);

		return node;
	}

	/**
	 * Replace empty Numeric node by Null node
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void removeLock(XMLNode node) {
		XMLNode item;
		if (node.getName().compareTo("Parameter") == 0) {
			for (int i = node.getNodeSize() - 1; i >= 0; i--) {
				item = node.getNode(i);
				if (item.getName().compareTo("Lock") == 0) {
					node.rmNode(i);
				}
			}
		}

		// next Node
		for (XMLNode n : node.getNodes()) {
			Converter_13.removeLock(n);
		}
	}

	/**
	 * Add new lock flag node to Parameter
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void addLock(XMLNode node) {
		XMLNode item;

		if (node.getName().compareTo("ParamField") == 0) {
			// add enable
			item = new XMLNode("Lock");
			item.setText("false");
			node.addNode(item);
		}

		// next Node
		for (XMLNode n : node.getNodes()) {
			Converter_13.addLock(n);
		}
	}
}
