/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9713 $
 * $Date: 2020-03-19 09:28:58 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.magopt.structure.converter;

import at.lcm.bu21.general.utils.XMLNode;

/**
 * Convert to V_0_19.
 * 
 * - Change FncSimulate from Matlab to Java <br>
 * - Remove SimRunning-flag
 * 
 * @author Werner Koppelsttter
 */
public class Converter_19 {

	/**
	 * Convert the given node
	 * 
	 * @param node
	 *            Data to convert
	 * @return Changed data node
	 */
	static XMLNode convert(XMLNode node) {
		// Convert Parameter
		Converter_19.changeSimulate(node);
		Converter_19.removeSimRunning(node);

		return node;
	}

	/**
	 * Change function simulate femag from matlab to java
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void changeSimulate(XMLNode node) {
		XMLNode item;

		if (node.getName().compareTo("ParamMatlabFunction") == 0) {
			item = node.getNode(1);
			if (item.getText().compareTo("FncSimulate") == 0) {
				// class name
				item.setText("femag.FncSimulate");

				// class path
				item = node.getNode(2);
				if (item.getText() != null) {
					item.setText("Femag\\Functions\\Java\\femag.jar");
				}

				item = node; // ParamMatlabFunction
				item.setName("ParamJavaFunction");
				item.setAttribute("object", "ParamJavaFunction");
				item = item.getNode(0); // ParamFunction
				item.setAttribute("object", "ParamJavaFunction");
				item = item.getNode(0); // ParamContainer
				item.setAttribute("object", "ParamJavaFunction");
				item = item.getNode(0); // Parameter
				item.setAttribute("object", "ParamJavaFunction");
			}
		}

		// next Node
		for (XMLNode n : node.getNodes()) {
			Converter_19.changeSimulate(n);
		}
	}

	/**
	 * Remove SimRunning-flag
	 * 
	 * @param node
	 *            Node to update
	 */
	private static void removeSimRunning(XMLNode node) {
		XMLNode item;

		// find SimRunning-flags in ParamFunction
		if (node.getName().compareTo("ParamFunction") == 0) {
			for (int k = node.getNodeSize() - 1; k >= 0; k--) {
				item = node.getNode(k);
				if (item.getName().compareTo("SimRunning") == 0) {
					node.rmNode(k);
				}
			}
		}

		// next Node
		for (XMLNode n : node.getNodes()) {
			Converter_19.removeSimRunning(n);
		}

	}
}
