/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.event;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;

import at.lcm.bu21.magopt.structure.event.Event;

/**
 * Stack-trace model for table
 * 
 * @author Werner Koppelsttter
 */
public class StackTraceModel extends AbstractTableModel {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Event to display */
	protected Event event = null;

	/**
	 * Set event to display
	 * 
	 * @param event
	 */
	public void setEvent(Event event) {
		this.event = event;

		// refresh table of model
		for (TableModelListener listener : this
				.getListeners(TableModelListener.class)) {
			if ((listener instanceof JTable)
					&& this.equals(((JTable) listener).getModel())) {
				listener.tableChanged(new TableModelEvent(this));
			}
		}

	}

	@Override
	public int getColumnCount() {
		return StackTraceColumn.values().length;
	}

	@Override
	public int getRowCount() {
		int len = 0;

		if (this.event != null) {
			len = this.event.getStack().length;
		}

		return len;
	}

	@Override
	public Object getValueAt(int row, int column) {
		Object value = null;
		StackTraceElement trace;

		if (this.event != null) {
			trace = this.event.getStack()[row];
			if (column == StackTraceColumn.CLASS.getValue()) {
				value = trace.getClassName();
			} else if (column == StackTraceColumn.FILE.getValue()) {
				value = trace.getFileName();
			} else if (column == StackTraceColumn.METHODE.getValue()) {
				value = trace.getMethodName();
			} else if (column == StackTraceColumn.LINE.getValue()) {
				value = Integer.toString(trace.getLineNumber());
			} else {
				value = trace;
			}
		}

		return value;
	}

	@Override
	public String getColumnName(int column) {
		for (StackTraceColumn e : StackTraceColumn.values()) {
			if (column == e.getValue()) {
				return e.getName();
			}
		}
		return null;
	}

	@Override
	public Class<?> getColumnClass(int column) {
		return String.class;
	}

}
