/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view;

import at.lcm.bu21.magopt.structure.IParamFunction;
import at.lcm.bu21.magopt.structure.IParamJavaFunction;
import at.lcm.bu21.magopt.structure.IParamMatlabFunction;
import at.lcm.bu21.magopt.structure.IParamPythonFunction;
import at.lcm.bu21.magopt.structure.IParameter;

public class PreviewView extends ParamBaseNoScroll implements ParamListener {

	private IParamFunction func;
	private ParamBaseNoScroll view;

	public PreviewView() {
	}

	public void setFunction(IParameter param) {
		if (param instanceof IParamFunction) {
			this.func = (IParamFunction) param;
		} else {
			this.func = null;
		}
		this.removeAll();
		if (this.view != null) {
			this.view.removeParamListener(this);
		}
		this.view = null;
		if (this.func != null) {
			if (this.func instanceof IParamJavaFunction) {
				this.view = new JavaPreviewView();
				((JavaPreviewView) this.view).setJavaFunction((IParamJavaFunction) this.func);
				this.view.addParamListener(this);
				this.add(this.view);
			} else if (this.func instanceof IParamPythonFunction) {
				this.view = new PythonPreviewView();
				((PythonPreviewView) this.view).setPythonFunction((IParamPythonFunction) this.func);
				this.view.addParamListener(this);
				this.add(this.view);
			} else if (this.func instanceof IParamMatlabFunction) {
				this.view = new MatlabPreviewView();
				((MatlabPreviewView) this.view).setMatlabFunction((IParamMatlabFunction) this.func);
				this.view.addParamListener(this);
				this.add(this.view);
			}
		}
	}

	@Override
	protected void update() {
		if (this.view != null) {
			this.view.update();
		}
	}

	@Override
	public void parameterChanged() {
		this.fireParamChanged();
	}
}
