/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.details;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.rmi.RemoteException;

import javax.swing.DefaultComboBoxModel;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import at.lcm.bu21.magopt.structure.IParamComboBox;
import magopt.gui.view.ParamBaseNoScroll;

public class ParamComboBoxView extends ParamBaseNoScroll
		implements ActionListener, FocusListener {
	private static final long serialVersionUID = 1L;

	private JTextField mName;
	private JTextField mDescription;
	private JTextField mComment;
	private JComboBox mData;
	private JTextField mUnit;
	private JTextField mTexString;
	private JTextField mType;
	private JList mListContent;
	private JButton mAdd;
	private JButton mRemove;
	private JCheckBox mConstant;
	private IParamComboBox mPComboBox;

	private boolean update = false;

	public ParamComboBoxView() {
		JPanel mainPane = new JPanel();
		this.setLayout(new BorderLayout());
		this.add(mainPane);

		GroupLayout layout = new GroupLayout(mainPane);
		mainPane.setLayout(layout);
		layout.setAutoCreateGaps(true);
		layout.setAutoCreateContainerGaps(true);

		/* name */
		JLabel lName = new JLabel("Name:");
		this.mName = new JTextField();
		this.mName.addActionListener(this);
		this.mName.addFocusListener(this);

		/* description */
		JLabel lDesc = new JLabel("Description:");
		this.mDescription = new JTextField();
		this.mDescription.addActionListener(this);
		this.mDescription.addFocusListener(this);

		/* comment */
		JLabel lComment = new JLabel("Comment:");
		this.mComment = new JTextField();
		this.mComment.addActionListener(this);
		this.mComment.addFocusListener(this);

		/* data */
		JLabel lData = new JLabel("Data:");
		this.mData = new JComboBox();
		this.mData.addActionListener(this);

		/* unit */
		JLabel lUnit = new JLabel("Unit:");
		this.mUnit = new JTextField();
		this.mUnit.addActionListener(this);
		this.mUnit.addFocusListener(this);

		/* tex string */
		JLabel lTexString = new JLabel("Tex String:");
		this.mTexString = new JTextField();
		this.mTexString.addActionListener(this);
		this.mTexString.addFocusListener(this);

		/* type */
		JLabel lType = new JLabel("Type:");
		this.mType = new JTextField();
		this.mType.addActionListener(this);
		this.mType.addFocusListener(this);

		/* constant flag */
		JLabel lConstant = new JLabel();
		this.mConstant = new JCheckBox("Constant");
		this.mConstant.addActionListener(this);

		/* list content */
		JLabel lListContent = new JLabel("List Content:");
		this.mListContent = new JList();
		JScrollPane spListContent = new JScrollPane(this.mListContent);

		/* add button (for list content) */
		this.mAdd = new JButton("Add");
		this.mAdd.addActionListener(this);

		/* remove button (list content) */
		this.mRemove = new JButton("Remove");
		this.mRemove.addActionListener(this);

		/* config group layout */
		layout.setHorizontalGroup(layout.createSequentialGroup().addGroup(layout
				.createParallelGroup(GroupLayout.Alignment.LEADING)
				.addComponent(lName).addComponent(lDesc).addComponent(lComment)
				.addComponent(lData).addComponent(lUnit)
				.addComponent(lTexString).addComponent(lType)
				.addComponent(lListContent).addComponent(lConstant))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.LEADING)
						.addComponent(this.mName)
						.addComponent(this.mDescription)
						.addComponent(this.mComment).addComponent(this.mData)
						.addComponent(this.mUnit).addComponent(this.mTexString)
						.addComponent(this.mType).addComponent(this.mConstant)
						.addGroup(layout.createSequentialGroup()
								.addComponent(spListContent)
								.addGroup(layout
										.createParallelGroup(
												GroupLayout.Alignment.LEADING)
										.addComponent(this.mAdd)
										.addComponent(this.mRemove)))));
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lName).addComponent(this.mName))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lDesc).addComponent(this.mDescription))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lComment).addComponent(this.mComment))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lData).addComponent(this.mData))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lUnit).addComponent(this.mUnit))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lTexString).addComponent(this.mTexString))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lType).addComponent(this.mType))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lConstant).addComponent(this.mConstant))
				.addGroup(layout
						.createParallelGroup(GroupLayout.Alignment.BASELINE)
						.addComponent(lListContent).addComponent(spListContent)
						.addGroup(layout.createSequentialGroup()
								.addComponent(this.mAdd)
								.addComponent(this.mRemove))));
		/* add & remove button should have the same horizontal size */
		layout.linkSize(SwingConstants.HORIZONTAL, this.mAdd, this.mRemove);
	}

	public void setComboBox(IParamComboBox pComboBox) {
		this.mPComboBox = pComboBox;
		this.update();
	}

	@Override
	public void focusGained(FocusEvent e) {
		/* do nothing */
	}

	@Override
	public void focusLost(FocusEvent e) {
		this.doWork(e.getSource());
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.doWork(e.getSource());
	}

	private void doWork(Object obj) {
		if (!this.update) {
			try {
				if (obj == this.mName) {
					this.mPComboBox.setName(this.mName.getText());
				} else if (obj == this.mDescription) {
					this.mPComboBox.setDescription(this.mDescription.getText());
				} else if (obj == this.mComment) {
					this.mPComboBox.setComment(this.mComment.getText());
				} else if (obj == this.mUnit) {
					this.mPComboBox.setUnit(this.mUnit.getText());
				} else if (obj == this.mTexString) {
					this.mPComboBox.setTexString(this.mTexString.getText());
				} else if (obj == this.mType) {
					this.mPComboBox.setType(this.mType.getText());
				} else if (obj == this.mConstant) {
					this.mPComboBox.setConstant(this.mConstant.isSelected());
				} else if (obj == this.mData) {
					this.mPComboBox
							.setComboIndex(this.mData.getSelectedIndex());
				} else if (obj == this.mAdd) {
					String strAdd = JOptionPane.showInputDialog(this,
							"New ComboBox Entry:", "<Name>");
					int curListIndex = this.mListContent.getSelectedIndex();
					String[] curList = this.mPComboBox.getList();
					String newList = "";

					for (int i = 0; i < curList.length; i++) {
						if (curListIndex == i) {
							newList = newList.concat(strAdd + ";");
						}
						newList = newList.concat(curList[i] + ";");
					}
					if (newList.endsWith(";") == true) {
						newList = newList.substring(0, newList.length() - 1);
					}
					this.mPComboBox.setComboList(newList);

				} else if (obj == this.mRemove) {
					int curListIndex = this.mListContent.getSelectedIndex();
					String[] curList = this.mPComboBox.getList();
					String newList = "";
					for (int i = 0; i < curList.length; i++) {
						if (i != curListIndex) {
							newList = newList.concat(curList[i] + ";");
						}
					}
					if (newList.endsWith(";") == true) {
						newList = newList.substring(0, newList.length() - 1);
					}
					this.mPComboBox.setComboList(newList);
				}

				this.update();
				this.fireParamChanged();
			} catch (RemoteException ex) {
				ex.printStackTrace();
			} catch (Exception ex) {
				try {
					this.mPComboBox.error(ex.getMessage(), ex.getStackTrace());
				} catch (RemoteException e) {
					e.printStackTrace();
				}
			}
		}
	}

	@Override
	protected void update() {
		try {
			this.mName.setText(this.mPComboBox.getName());
			this.mDescription.setText(this.mPComboBox.getDescription());
			this.mComment.setText(this.mPComboBox.getComment());
			// FIXME update
			this.update = true;
			this.mData.setModel(
					new DefaultComboBoxModel(this.mPComboBox.getList()));
			this.mData.setSelectedIndex(this.mPComboBox.getComboIndex());
			this.update = false;
			this.mData.setEditable(false);
			this.mUnit.setText(this.mPComboBox.getUnit());
			this.mTexString.setText(this.mPComboBox.getTexString());
			this.mType.setText(this.mPComboBox.getType());
			this.mListContent.setListData(this.mPComboBox.getList());
			this.mListContent.setSelectedValue(this.mData.getSelectedItem(),
					true);
			this.mConstant.setSelected(this.mPComboBox.getConstant());

			this.mData.setEnabled(!this.mPComboBox.getReadOnly());
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}
}
