/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9749 $
 * $Date: 2020-04-10 16:00:50 +0200 (Fri, 10 Apr 2020) $
 */
package magopt.gui.view.details;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.lang.reflect.InvocationTargetException;
import java.rmi.RemoteException;

import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import at.lcm.bu21.general.dtypes.TBoolean;
import at.lcm.bu21.general.dtypes.TData;
import at.lcm.bu21.general.dtypes.TNull;
import at.lcm.bu21.general.dtypes.TNumeric;
import at.lcm.bu21.general.dtypes.TString;
import at.lcm.bu21.magopt.structure.DependDirection;
import at.lcm.bu21.magopt.structure.IParamLinkedField;
import at.lcm.bu21.magopt.structure.ParamField;
import magopt.gui.helpers.DataEditor;
import magopt.gui.view.ParamBaseNoScroll;
import magopt.gui.view.port.PortView;

/**
 * Displays ParamLinkedField specific Data:<br>
 * - Data<br>
 * - Unit<br>
 * - Tex String<br>
 * - Type - Imports
 * 
 * @author Aigner Markus
 * @author Werner Koppelsttter
 */
public class ParamLinkedFieldView extends ParamBaseNoScroll implements ActionListener, FocusListener {

	private JTextField mName;
	private JTextField mDescription;
	private JTextField mComment;

	protected JTextField mData;
	protected JTextField mUnit;
	protected JTextField mTexString;
	protected JTextField mType;

	protected JButton bEdit;
	protected JButton bShow;

	protected boolean show;

	protected PortView mImports;

	private IParamLinkedField mPLinkedField;

	private int ret;

	public ParamLinkedFieldView() {
		JPanel mainPane = new JPanel();
		this.setLayout(new BorderLayout());
		this.add(mainPane);

		GroupLayout layout = new GroupLayout(mainPane);
		mainPane.setLayout(layout);
		layout.setAutoCreateGaps(true);
		layout.setAutoCreateContainerGaps(true);

		/* name */
		JLabel lName = new JLabel("Name:");
		this.mName = new JTextField();
		this.mName.addActionListener(this);
		this.mName.addFocusListener(this);

		/* description */
		JLabel lDesc = new JLabel("Description:");
		this.mDescription = new JTextField();
		this.mDescription.addActionListener(this);
		this.mDescription.addFocusListener(this);

		/* comment */
		JLabel lComment = new JLabel("Comment:");
		this.mComment = new JTextField();
		this.mComment.addActionListener(this);
		this.mComment.addFocusListener(this);

		/* data */
		JLabel lData = new JLabel("Data:");
		this.mData = new JTextField();
		this.mData.addActionListener(this);
		this.mData.addFocusListener(this);

		/* edit */
		this.bEdit = new JButton("Edit");
		this.bEdit.addActionListener(this);
		// this.bEdit.addFocusListener(this);

		/* show */
		this.bShow = new JButton("Show");
		this.bShow.addActionListener(this);
		// this.bShow.addFocusListener(this);

		/* unit */
		JLabel lUnit = new JLabel("Unit:");
		this.mUnit = new JTextField();
		this.mUnit.addActionListener(this);
		this.mUnit.addFocusListener(this);

		/* tex string */
		JLabel lTexString = new JLabel("Tex String:");
		this.mTexString = new JTextField();
		this.mTexString.addActionListener(this);
		this.mTexString.addFocusListener(this);

		/* type */
		JLabel lType = new JLabel("Type:");
		this.mType = new JTextField();
		this.mType.addActionListener(this);
		this.mType.addFocusListener(this);

		/* imports */
		JLabel lImports = new JLabel("Imports");
		this.mImports = new PortView();
		// this.mImports.addActionListener(this);

		/* config group layout */
		layout.setHorizontalGroup(layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addComponent(lName)
						.addComponent(lDesc).addComponent(lComment).addComponent(lData).addComponent(lUnit)
						.addComponent(lTexString).addComponent(lType).addComponent(lImports))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addComponent(this.mName)
						.addComponent(this.mDescription).addComponent(this.mComment).addComponent(this.mData)
						.addGroup(layout.createSequentialGroup().addComponent(this.bEdit).addComponent(this.bShow))
						.addComponent(this.mUnit).addComponent(this.mTexString).addComponent(this.mType)
						.addComponent(this.mImports))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addComponent(this.bShow)));
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(lName)
						.addComponent(this.mName))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(lDesc)
						.addComponent(this.mDescription))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(lComment)
						.addComponent(this.mComment))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(lData)
						.addComponent(this.mData))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(this.bEdit)
						.addComponent(this.bShow))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(lUnit)
						.addComponent(this.mUnit))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(lTexString)
						.addComponent(this.mTexString))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(lType)
						.addComponent(this.mType))
				.addGroup(layout.createParallelGroup(GroupLayout.Alignment.BASELINE).addComponent(lImports)
						.addComponent(this.mImports)));
	}

	@Override
	public void focusGained(FocusEvent e) {
		/* do nothing */
	}

	@Override
	public void focusLost(FocusEvent e) {
		this.doWork(e.getSource());
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		this.doWork(e.getSource());
	}

	private void doWork(Object obj) {
		new Thread() {
			@Override
			public void run() {

				try {
					if (obj == ParamLinkedFieldView.this.mName) {
						ParamLinkedFieldView.this.mPLinkedField.setName(ParamLinkedFieldView.this.mName.getText());
					} else if (obj == ParamLinkedFieldView.this.mDescription) {
						ParamLinkedFieldView.this.mPLinkedField
								.setDescription(ParamLinkedFieldView.this.mDescription.getText());
					} else if (obj == ParamLinkedFieldView.this.mComment) {
						ParamLinkedFieldView.this.mPLinkedField
								.setComment(ParamLinkedFieldView.this.mComment.getText());
					} else if (obj == ParamLinkedFieldView.this.mData) {
						JTextField field = (JTextField) obj;
						String str = field.getText();
						ParamLinkedFieldView.this.mPLinkedField.setData(TData.createData(str));
						ParamLinkedFieldView.this.updateDataEditable();
					} else if (obj == ParamLinkedFieldView.this.mUnit) {
						if (!ParamLinkedFieldView.this.mPLinkedField.getReadOnly()) {
							synchronized (ParamLinkedFieldView.this) {
								if ((ParamLinkedFieldView.this.mUnit.getText() != null)
										&& (!ParamLinkedFieldView.this.mUnit.getText().isEmpty())
										&& (ParamLinkedFieldView.this.mPLinkedField.getUnit() != null)
										&& (!ParamLinkedFieldView.this.mPLinkedField.getUnit().isEmpty())
										&& !ParamLinkedFieldView.this.mPLinkedField.getUnit()
												.equals(ParamLinkedFieldView.this.mUnit.getText())) {

									SwingUtilities.invokeAndWait(new Runnable() {
										@Override
										public void run() {
											ParamLinkedFieldView.this.ret = JOptionPane.showConfirmDialog(
													ParamLinkedFieldView.this,
													"Do you want to convert value to new unit?\n" + "Please confirm!",
													"Data conversion confirmation", JOptionPane.YES_NO_OPTION,
													JOptionPane.WARNING_MESSAGE, null);
										}
									});

									ParamLinkedFieldView.this.mPLinkedField.setUnit(
											ParamLinkedFieldView.this.mUnit.getText(),
											ParamLinkedFieldView.this.ret != JOptionPane.NO_OPTION);
								} else {
									ParamLinkedFieldView.this.mPLinkedField
											.setUnit(ParamLinkedFieldView.this.mUnit.getText());
								}
							}
						} else {
							ParamLinkedFieldView.this.mPLinkedField.setUnit(ParamLinkedFieldView.this.mUnit.getText(),
									true);
						}
					} else if (obj == ParamLinkedFieldView.this.mTexString) {
						ParamLinkedFieldView.this.mPLinkedField
								.setTexString(ParamLinkedFieldView.this.mTexString.getText());
					} else if (obj == ParamLinkedFieldView.this.mType) {
						ParamLinkedFieldView.this.mPLinkedField.setType(ParamLinkedFieldView.this.mType.getText());
					} else if (obj == ParamLinkedFieldView.this.bEdit) {
						// FIXME RMI
						new DataEditor((ParamField) ParamLinkedFieldView.this.mPLinkedField);
					} else if (obj == ParamLinkedFieldView.this.bShow) {
						ParamLinkedFieldView.this.show = true;
						ParamLinkedFieldView.this.bShow.setEnabled(!ParamLinkedFieldView.this.show);
					}
				} catch (

				RemoteException ex) {
					ex.printStackTrace();
				} catch (Exception ex) {
					try {
						ParamLinkedFieldView.this.mPLinkedField.error(ex.getMessage(), ex.getStackTrace());
					} catch (RemoteException e) {
						e.printStackTrace();
					}
				}

				try {
					SwingUtilities.invokeAndWait(new Runnable() {

						@Override
						public void run() {
							ParamLinkedFieldView.this.update();
							ParamLinkedFieldView.this.fireParamChanged();
						}
					});
				} catch (InvocationTargetException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}
		}.start();
	}

	public void setLinkedField(IParamLinkedField pLinkedField) {
		this.mPLinkedField = pLinkedField;
		try {
			this.show = !this.mPLinkedField.getUncertain();
			this.bShow.setEnabled(!this.show);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		this.update();
	}

	@Override
	protected void update() {
		try {
			this.mName.setText(this.mPLinkedField.getName());
			this.mDescription.setText(this.mPLinkedField.getDescription());
			this.mComment.setText(this.mPLinkedField.getComment());
			if (this.show) {
				this.mData.setText(this.mPLinkedField.getData().toString());
			} else {
				this.mData.setText("[Uncertain]");
			}
			this.mUnit.setText(this.mPLinkedField.getUnit());
			this.mTexString.setText(this.mPLinkedField.getTexString());
			this.mType.setText(this.mPLinkedField.getType());
			this.mData.setEditable(!this.mPLinkedField.getReadOnly());
			this.mImports.setDepend(this.mPLinkedField.getDepend(), DependDirection.IMPORT);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	private void updateDataEditable() throws RemoteException {
		TData data = this.mPLinkedField.getData();
		if (this.mPLinkedField.getReadOnly()) {
			this.mData.setEditable(false);
		} else {
			if (data.getClass().equals(TNumeric.class)) {
				this.mData.setEditable(data.toString().equals(((TNumeric) data).toFullString()));
			} else if (data.getClass().equals(TString.class)) {
				this.mData.setEditable(data.toString().equals("'" + ((TString) data).getString() + "'"));
			} else if (data.getClass().equals(TBoolean.class)) {
				this.mData.setEditable(true);
			} else if (data.getClass().equals(TNull.class)) {
				this.mData.setEditable(true);
			} else {
				this.mData.setEditable(false);
			}
		}
	}
}
