/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.port;

import java.rmi.RemoteException;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;

import at.lcm.bu21.magopt.structure.DependDirection;
import at.lcm.bu21.magopt.structure.IDepend;
import at.lcm.bu21.magopt.structure.IDependItem;

/**
 * Table model for import- and export table types.
 * 
 * @author Aigner Markus
 * @author Werner Koppelsttter
 */
public class PortModel extends AbstractTableModel {
	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** Depends object */
	private IDepend depend = null;

	/** Direction of depends */
	private DependDirection direction = null;

	/**
	 * Set items to display depends
	 * 
	 * @param depend
	 *            Depend-object
	 * @param direction
	 *            Direction of depend
	 */
	public void setDepend(IDepend depend, DependDirection direction) {
		this.depend = depend;
		this.direction = direction;

		// refresh table of model
		for (TableModelListener listener : this
				.getListeners(TableModelListener.class)) {
			if ((listener instanceof JTable)
					&& this.equals(((JTable) listener).getModel())) {
				listener.tableChanged(new TableModelEvent(this));
			}
		}
	}

	@Override
	public int getColumnCount() {
		return PortColumn.values().length;
	}

	@Override
	public int getRowCount() {
		int len = 0;

		if (this.depend != null) {
			try {
				len = this.depend.getDependsLength(this.direction);
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}

		return len;
	}

	@Override
	public Object getValueAt(int row, int column) {
		Object obj = null;
		IDependItem item;

		if (this.depend != null) {
			try {
				item = this.depend.getDepend(this.direction, row);

				if (column == PortColumn.VARIABLE.getValue()) {
					obj = item.getVarName();
				} else if (column == PortColumn.PARAMETER.getValue()) {
					obj = item.getParam();
				} else if (column == PortColumn.HANDLE.getValue()) {
					obj = (item.getHandle() != null);
				} else {
					obj = item;
				}
			} catch (RemoteException e) {
				e.printStackTrace();
			}
		}

		return obj;
	}

	@Override
	public String getColumnName(int column) {
		for (PortColumn e : PortColumn.values()) {
			if (column == e.getValue()) {
				return e.getName();
			}
		}
		return null;
	}

	@Override
	public Class<?> getColumnClass(int column) {
		Class<?> type = String.class;

		if (column == PortColumn.HANDLE.getValue()) {
			type = Boolean.class;
		}

		return type;
	}

	@Override
	public boolean isCellEditable(int row, int column) {
		boolean edit = false;

		if (column == PortColumn.PARAMETER.getValue()) {
			edit = true;
		}

		return edit;
	}

	@Override
	public void setValueAt(Object value, int row, int col) {
		if (this.depend != null) {
			if (col == PortColumn.PARAMETER.getValue()) {
				try {
					this.depend.setAssignment(
							(String) this.getValueAt(row,
									PortColumn.VARIABLE.getValue()),
							value.toString(), this.direction);
				} catch (RemoteException e) {
					e.printStackTrace();
				}
				this.fireTableDataChanged();
			}
		}
	}
}
