/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.gui.view.summary;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.KeyEvent;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import at.lcm.bu21.magopt.structure.IParamContainer;
import at.lcm.bu21.magopt.structure.IParameter;
import magopt.gui.view.ParamBaseScroll;

/**
 * Displays container summary specific data.
 * 
 * @author Aigner Markus
 * @author Werner Koppelsttter
 */
public class SummaryView extends ParamBaseScroll {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** ParamContainer to display */
	private IParamContainer mParamContainer;

	/** Table to display container */
	// public for matlab gui
	public JTable table;

	/** Import-model */
	private SummaryModel model;

	/** Main panel */
	private JPanel mainPanel;

	/**
	 * Create container-summary view
	 */
	public SummaryView() {
		JScrollPane panelTable;

		this.mainPanel = new JPanel();
		this.add(this.mainPanel);
		this.mainPanel.setOpaque(true);
		this.mainPanel.setVisible(false);
		this.mainPanel.setLayout(new BorderLayout());

		this.mainPanel.setPreferredSize(this.getPreferredSize());
		this.mainPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		panelTable = new JScrollPane();
		this.mainPanel.add(panelTable);
		this.table = new JTable();
		panelTable.setViewportView(this.table);
		this.model = new SummaryModel();
		this.table.setModel(this.model);
		this.table.getTableHeader().setReorderingAllowed(false);
		this.table.setColumnSelectionAllowed(true);
		this.table.setRowSelectionAllowed(true);
		this.table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		this.table.setDefaultRenderer(Object.class, new SummaryRenderer());
		// Select test by enter the editor
		this.table.setDefaultEditor(Object.class, new DefaultCellEditor(new JTextField()) {

			/** Serialization version number */
			private static final long serialVersionUID = 1L;

			@Override
			public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row,
					int column) {
				Component c;
				c = super.getTableCellEditorComponent(table, value, isSelected, row, column);
				((JTextField) c).selectAll();
				return c;
			}
		});
		// Change action of enter to action of tab (change to next right cell)
		InputMap im = this.table.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		im.put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), im.get(KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0)));

		this.model.addTableModelListener(new TableModelListener() {
			@Override
			public void tableChanged(TableModelEvent e) {
				SummaryView.this.fireParamChanged();
			}
		});

		// define size
		this.table.getColumnModel().getColumn(SummaryColumn.TYPE.getValue()).setPreferredWidth(40);
		this.table.getColumnModel().getColumn(SummaryColumn.NAME.getValue()).setPreferredWidth(100);
		this.table.getColumnModel().getColumn(SummaryColumn.DESCRIPTION.getValue()).setPreferredWidth(200);
		this.table.getColumnModel().getColumn(SummaryColumn.VALUE.getValue()).setPreferredWidth(90);
		this.table.getColumnModel().getColumn(SummaryColumn.UNIT.getValue()).setPreferredWidth(70);

		this.table.getColumnModel().getColumn(SummaryColumn.TYPE.getValue()).setMaxWidth(40);

	}

	/**
	 * Sets new ParamContainer.
	 * 
	 * @param param
	 *            - ParamContainer
	 */
	public void setContainer(IParameter param) {
		if (this.table.isEditing()) {
			this.table.getCellEditor().cancelCellEditing();
		}
		if (param instanceof IParamContainer) {
			this.mParamContainer = (IParamContainer) param;
		} else {
			this.mParamContainer = null;
		}
		this.mainPanel.setVisible(this.mParamContainer != null);
		this.update();
	}

	@Override
	protected void update() {
		this.model.setContainer(this.mParamContainer);
	}
}
