/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9714 $
 * $Date: 2020-03-19 09:37:13 +0100 (Thu, 19 Mar 2020) $
 */
package magopt.utils;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;

import javax.swing.JFrame;
import javax.swing.JProgressBar;

/**
 * @author gweidenholzer
 * 
 */
public class ProgressBar {

	JFrame f = null;
	JProgressBar progressBar = null;
	int max;

	private class ColorMap {

		Color c1, c2;

		ColorMap(Color c1, Color c2) {
			this.c1 = c1;
			this.c2 = c2;
		}

		Color getColor(float x) {

			float[] c1_ = this.c1.getRGBComponents(new float[4]);
			float[] c2_ = this.c2.getRGBComponents(new float[4]);

			return new Color(c1_[0] + (x * (c2_[0] - c1_[0])), c1_[1]
					+ (x * (c2_[1] - c1_[1])), c1_[2] + (x * (c2_[2] - c1_[2])));
		}
	}

	ColorMap colmap = new ColorMap(Color.red, Color.green);

	/**
	 * 
	 * @param title
	 *            Title of progress bar
	 * @param max
	 */
	public ProgressBar(String title, int max) {

		this.f = new JFrame(title);

		this.max = max;

		// Make a Progress Bar
		this.progressBar = new JProgressBar(0, max);
		this.progressBar.setValue(0);
		this.progressBar.setStringPainted(true);
		this.f.setLayout(new BorderLayout());
		this.f.add(this.progressBar);
		this.f.pack();
		this.f.setSize(400, 65);

		// Get the size of the screen
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();

		// Determine the new location of the window
		int w = this.f.getSize().width;
		int h = this.f.getSize().height;
		int x = (dim.width - w) / 2;
		int y = (dim.height - h) / 2;

		// Move the window
		this.f.setLocation(x, y);

		this.setValue(0);

		this.f.setVisible(true);

	}

	/**
	 * Set current value
	 * 
	 * @param r
	 */
	public void setValue(int r) {

		this.progressBar.setValue(r);
		this.progressBar.setForeground(this.colmap.getColor((float) r
				/ this.max));
		this.progressBar.repaint();

	}

	/**
	 * hide bar
	 */
	public void invisible() {
		this.f.setVisible(false);
	}

	public void dispose() {
		this.f.dispose();
	}

}
