/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 247 $
 * $Date: 2020-03-19 10:13:52 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.optim.algorithm.grid;

import java.awt.Dimension;

import javax.swing.GroupLayout;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.LayoutStyle;

import at.lcm.bu21.optim.algorithm.JPanelSolver;

/**
 * Panel for settings of grid optimizer
 * 
 * @author Werner Koppelsttter
 */
public class GridPanel extends JPanelSolver {

	/** Serialization version number */
	private static final long serialVersionUID = 1L;

	/** GridReal optimizer with setting values */
	private Grid solver;

	/**
	 * Size of possible archive
	 */
	private JTextField txtArchiveSize;

	/**
	 * Create panel. Used by WindowsBuilder
	 */
	private GridPanel() {
		this.initComponents();
	}

	/**
	 * Creates new setting-panel for Grid2Real
	 * 
	 * @param solver
	 *            Solver with setting values
	 */
	public GridPanel(Grid solver) {
		this();
		this.solver = solver;

		this.txtArchiveSize.setText(Integer.toString(this.solver
				.getArchiveSize()));
	}

	@Override
	public void refreshEnabled() {
	}

	/**
	 * Initialize components
	 */
	private void initComponents() {
		JLabel labArchiveSizeLabel, labArchiveSizeUnit;

		labArchiveSizeLabel = new JLabel();
		this.txtArchiveSize = new JTextField();
		labArchiveSizeUnit = new JLabel();

		labArchiveSizeLabel.setText("Archive size:");

		this.txtArchiveSize.setEnabled(false);
		this.txtArchiveSize.setPreferredSize(new Dimension(100, 20));

		labArchiveSizeUnit.setText("Individuals");

		GroupLayout layout = new GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addGroup(
				layout.createSequentialGroup()
						.addContainerGap()
						.addComponent(labArchiveSizeLabel)
						.addGap(99, 99, 99)
						.addComponent(this.txtArchiveSize,
								GroupLayout.PREFERRED_SIZE, 47,
								GroupLayout.PREFERRED_SIZE)
						.addPreferredGap(
								LayoutStyle.ComponentPlacement.UNRELATED)
						.addComponent(labArchiveSizeUnit,
								GroupLayout.PREFERRED_SIZE, 59,
								GroupLayout.PREFERRED_SIZE)
						.addContainerGap(170, Short.MAX_VALUE)));
		layout.setVerticalGroup(layout.createParallelGroup(
				GroupLayout.Alignment.LEADING).addGroup(
				layout.createSequentialGroup()
						.addContainerGap()
						.addGroup(
								layout.createParallelGroup(
										GroupLayout.Alignment.BASELINE)
										.addComponent(labArchiveSizeLabel)
										.addComponent(this.txtArchiveSize,
												GroupLayout.PREFERRED_SIZE,
												GroupLayout.DEFAULT_SIZE,
												GroupLayout.PREFERRED_SIZE)
										.addComponent(labArchiveSizeUnit))
						.addContainerGap(269, Short.MAX_VALUE)));
	}
}
