/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 247 $
 * $Date: 2020-03-19 10:13:52 +0100 (Thu, 19 Mar 2020) $
 */
package at.lcm.bu21.optim.core.mapping.utils;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import at.lcm.bu21.optim.core.Individual;
import at.lcm.bu21.optim.core.mapping.MappingManager;

public class AWSAThread extends Thread {

	private MappingManager parentMappingManager;

	private String filePrefix;

	public AWSAThread(MappingManager parentMappingManager, String filePrefix) {
		this.parentMappingManager = parentMappingManager;
		this.filePrefix = filePrefix;
	}

	@Override
	public void run() {

		// Individual[] individuals = this.getProblem().getArchive().getMemory()
		// .getIndividual();
		List<Individual> indsToAnalyze = new ArrayList<Individual>();
		List<Integer> savePointSizes = new ArrayList<Integer>();

		int maxIndsToAnalyze = 100000;
		int minSavePointNr = 0;
		int maxSavePointNr = this.parentMappingManager.getProblem().getNumRealSavePoints();

		for (int savePointNr = minSavePointNr; savePointNr < maxSavePointNr; savePointNr++) {
			Individual[] individuals = this.parentMappingManager.getProblem().getSavePoint(savePointNr)
					.getLocalPareto();

			System.out.println("Save point " + savePointNr + ": " + individuals.length);

			savePointSizes.add(individuals.length);

			for (int i = 0; i < individuals.length; i++) {
				if (!individuals[i].isError()) {
					indsToAnalyze.add(individuals[i]);
					maxIndsToAnalyze--;
					if (maxIndsToAnalyze == 0) {
						break;
					}
				}
			}

			if (maxIndsToAnalyze == 0) {
				break;
			}
		}

		System.out.println("Memory individuals to analyze: " + indsToAnalyze.size() + " from: "
				+ (maxSavePointNr - minSavePointNr) + " save points!");
		int i = 0;
		int cGen = 0;
		int ci = 0;

		String genInfoFileName = "d:\\\\Results\\SA_test\\" + this.filePrefix + "genInfo.csv";
		try {
			File g = new File(genInfoFileName);
			File dir = new File(g.getParent());
			if (!dir.exists() && !dir.mkdirs()) {
				throw new IOException("Could not create directory path: " + g.getParent());
			}

			if (!g.exists()) {
				g.createNewFile();
			}
			BufferedWriter bw = new BufferedWriter(new FileWriter(genInfoFileName, false));

			bw.write(savePointSizes.get(cGen) + "\n");

			bw.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

		for (Individual ind : indsToAnalyze) {
			i++;
			ci++;
			System.out.println("-----SA started for individual nr. " + i + " out of " + indsToAnalyze.size() + "-----");
			SensitivityAnalysisHolder tempHolder = new SensitivityAnalysisHolder(this.parentMappingManager, ind);
			tempHolder.createLocalSensitivityIndividuals();
			tempHolder.createGlobalSensitivityIndividuals();
			tempHolder.updateIndividualsCount();
			tempHolder.evaluateSensitivityUsingMappings();
			tempHolder.postProcessMappingBasedSensitivityInformation();

			this.parentMappingManager.updateAWSAProgress((i * 1.0) / indsToAnalyze.size());

			for (String targetName : this.parentMappingManager.getSensitivityTargetNames()) {

				String fileName = "d:\\\\Results\\SA_test\\" + this.filePrefix + targetName + ".csv";

				double real = tempHolder.getBaseIndividual("FE").getResult(targetName);
				double min = tempHolder.getGlobalSensitivityAbsoluteMinValueToDisplayDouble(targetName);
				double max = tempHolder.getGlobalSensitivityAbsoluteMaxValueToDisplayDouble(targetName);
				double diff = max - min;
				double loss = Math.max(max - real, 0.0);

				try {
					File f = new File(fileName);
					File dir = new File(f.getParent());
					if (!dir.exists() && !dir.mkdirs()) {
						throw new IOException("Could not create directory path: " + f.getParent());
					}

					if (!f.exists()) {
						f.createNewFile();
					}
					BufferedWriter bw = new BufferedWriter(new FileWriter(fileName, true));

					String s = cGen + "," + real + "," + min + "," + max + "," + diff + "," + loss;
					bw.write(s + "\n");

					bw.close();
				} catch (Exception e) {
					e.printStackTrace();
				}

				System.out.println(targetName + " GEN=" + cGen + " REAL=" + real + " MIN=" + min + " MAX=" + max
						+ " DIFF=" + diff + " LOSS=" + loss);
			}
			System.out.println("-------------------------------------------------------");

			if (ci == savePointSizes.get(cGen)) {
				ci = 0;
				cGen++;
				if (cGen < savePointSizes.size()) {
					try {
						File g = new File(genInfoFileName);
						File dir = new File(g.getParent());
						if (!dir.exists() && !dir.mkdirs()) {
							throw new IOException("Could not create directory path: " + g.getParent());
						}

						if (!g.exists()) {
							g.createNewFile();
						}
						BufferedWriter bw = new BufferedWriter(new FileWriter(genInfoFileName, true));

						bw.write(savePointSizes.get(cGen) + "\n");

						bw.close();
					} catch (Exception e) {
						e.printStackTrace();
					}
				}
			}

		}

		this.parentMappingManager.handleAWSASucessfulRun();

	}

}
