/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 10077 $
 * $Date: 2020-12-09 08:25:37 +0100 (Mi., 09 Dez 2020) $
 */
package at.lcm.bu21.optim.core.plot;

import java.lang.reflect.Method;

import at.lcm.bu21.optim.core.Problem;

public class WebPlotInterface {

	public static Class<?> server = null;

	static {
		try {
			WebPlotInterface.server = Class.forName("at.lcm.symspace.optimizer.rest.plot.RESTWebPlotServer");
		} catch (ClassNotFoundException e) {
		}

	}

	public static void setProblem(Problem problem, boolean plotVariables, PlotJson.ResultType plotRestuls) {
		Method m;

		if (WebPlotInterface.exists()) {
			try {
				m = WebPlotInterface.server.getMethod("setProblem", Problem.class, boolean.class,
						PlotJson.ResultType.class);
				m.invoke(null, problem, plotVariables, plotRestuls);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public static void browse() {
		Method m;

		if (WebPlotInterface.exists()) {
			try {
				m = WebPlotInterface.server.getMethod("browse");
				m.invoke(null);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public static void startServer() {
		Method m;

		if (WebPlotInterface.exists()) {
			try {
				m = WebPlotInterface.server.getMethod("startServer");
				m.invoke(null);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public static void stopServer() {
		Method m;

		if (WebPlotInterface.exists()) {
			try {
				m = WebPlotInterface.server.getMethod("stopServer");
				m.invoke(null);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public static boolean isRunning() {
		boolean running = false;
		Method m;

		if (WebPlotInterface.exists()) {
			try {
				m = WebPlotInterface.server.getMethod("isRunning");
				running = (Boolean) m.invoke(null);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}

		return running;
	}

	public static boolean isAvailable() {
		boolean available = false;
		Method m;

		if (WebPlotInterface.exists()) {
			try {
				m = WebPlotInterface.server.getMethod("isAvailable");
				available = (Boolean) m.invoke(null);
			} catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
			}
		}

		return available;
	}

	public static boolean exists() {
		return WebPlotInterface.server != null;
	}

}
