#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 10007 $
$LastChangedDate:: 2020-09-30 13:22:48 +0200#$

This file is part of SyMSpace.

Copyright (c) 2019, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import numpy as np
from symspace.parampythonfnc import ParamPythonFnc
# from symspace.decorators import debugexternal, remotecompute


# @remotecompute('(PYTHON >= \"3.5.3\")')
# @remotecompute('(PYTHON >= \"3.5.3\")', variation=['a', 'b'])
# @debugexternal
class FncPyTemplate(ParamPythonFnc):
    """ Template Python Function
    
    SyMSpace template class, which shows the basic implementation of a Python function.

    """

    def importVars(self):
        """Description of the import variables

        Returns a list of tuples, which defines the import variables.
        Each tuple consists of multiple entries in the following order:

        (Name : string, Description : string, Unit : string, Optional : bool, Type / Dimension : string)

        """
        return [('a', 'Adjacent leg', 'mm', False, 'Numeric / [1 x 1]'),
                ('b', 'Opposite leg', 'mm', False, 'Numeric / [1 x 1]')]

    def exportVars(self):
        """Description of the export variables

        Returns a list of tuples, which defines the export variables.
        Each tuple consists of multiple entries in the following order:

        (Name : string, Description : string, Unit : string, Optional : bool, Type / Dimension : string)

        """
        return [('c', 'Hypotenuse', 'mm', False, 'Numeric / [1 x 1]')]

    def run(self, run_args):
        """Run method of the function, which calculates the result

        Parameters
        ----------
        run_args : dict
                Arguments of caller (Control flags)
                {RemoteCompute=false, RunSim=false, Optimize=false}

        Returns
        -------
        status : int
            If status == 0 the function regenerated successfully
            Else the function update is not completed OR failed

        """

        '''import variables from ports'''
        d = self.importValues()
        a = d['a']
        b = d['b']

        '''calculate the hypotenuse'''
        c = np.sqrt(a ** 2. + b ** 2.)
        d['c'] = c
        
        '''export variables to ports'''
        self.exportValues(d)

        return 0

    # def preview(self, width, height):
    #     """Preview method of the function
    #
    #     Returns a list of tuples, which defines the preview frame.
    #     Each tuple represents a tab in the preview frame
    #     Each tuple consists of at least three entries:
    #
    #     (Name : string, Content Type : string, Content : object)
    #
    #     Parameters
    #     ----------
    #     width : int
    #         Frame width
    #     height : int
    #         Frame height
    #
    #     Returns
    #     -------
    #     list
    #
    #
    #     """
    #     '''import variables from ports'''
    #     d = self.importValues()
    #     '''import previously calculated export variables from ports'''
    #     self.revertExportValues(d)
    #
    #     return [('PNG', 'image/png', fig_png),
    #             ('HTML', 'text/html', fig_html),
    #             ('JAVA', 'application/java', fig_java)
    #             ]

    # def postJob(self, d: dict, run_args: dict):
    #     """
    #     Run this method after running the main method of the subclass (run, runJob)
    #
    #     e.g. to change the structure, value or type of export variables before exporting them
    #
    #     Parameters
    #     ----------
    #     run_args : dict
    #     d : dict
    #         Contains importVars and exportVars
    #     """
    #     self.convertVariantsType(d)
