# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 10096 $
$LastChangedDate:: 2021-01-21 16:08:58 +0100#$

This file is part of SyMSpace.

Copyright (c) 2020, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import os
from symspace.generic_gui_qt import GUIFormThread
from symspace.console.script_utils import ScriptUtils
from symspace.wrapper.symoptwrapper import SymXLSWrite, SymArchive


def main(params: dict):
    project = ScriptUtils.selectProjectFromParams(params)
    if project is not None:
        xls_path_default = os.path.join(project.getDirectory(), 'Optimizer_Export')
        prj_file = project.getFile(get_abs_path=True)
        individual_types = ['all', 'valid', 'no_error', 'error', 'pareto']
        archive_problem_list = ScriptUtils.getProblemsList(prj_file)
        archive_problem_list.append('all')

        form_title = 'Export Memory to Excel file'
        form_elements = [
            ('Select Excel export directory', 'browse_folder', xls_path_default),
            ('Archive//Problem Name', 'combobox', archive_problem_list),
            ('Individual Type', 'combobox', individual_types),
        ]
        form = GUIFormThread(form_title, form_elements)

        form.start()
        if form.accept:
            xls_dir = form.result['Select Excel export directory']
            individual_type = form.result['Individual Type']
            archive_problem = form.result['Archive//Problem Name']

            if archive_problem == 'all':
                SymXLSWrite.exportMultipleXLSReports(prj_file, xls_dir, archive_names=None, problem_names=None,
                                                     individual_type=individual_type)
            else:
                archive_name, problem_name = ScriptUtils.parseArchiveProblemName(archive_problem)

                archive = SymArchive.load(prj_file, archive_name)
                problem = archive.readProblem(problem_name)
                file = os.path.join(xls_dir, SymXLSWrite.getStandardFileName(archive_name, problem_name, individual_type))

                SymXLSWrite.writeXLSReportMemory(problem, file, individual_type)

            print('Update finished')
        else:
            print('Cancelled by user')


if __name__ == '__main__':
    """
    This script exports Optimization individuals (in Memory) to  Excel spread sheet(s)

    This script needs to be executed in the SyMSpace Python Console.
    """
    _params = locals()
    ScriptUtils.run(main, _params)
