#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision$
$LastChangedDate::                           $

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import os
import stat
import shutil
import fileinput


def copytree(src: str, dst: str, symlinks: bool = False, ignore=None):
    """
    Copies directory with files recursively

    Parameters
    ----------
    src : str
        Source path
    dst : str
        Destination path
    symlinks : bool
    ignore

    """

    if not os.path.exists(dst):
        os.makedirs(dst)
    for item in os.listdir(src):
        # Skip Subversion directories
        if item == '.svn':
            continue
        # Skip __pycache__ directory
        if item == '__pycache__':
            continue
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            copytree(s, d, symlinks, ignore)
        else:
            if not os.path.exists(d) or os.stat(s).st_mtime - os.stat(d).st_mtime > 1:
                shutil.copy2(s, d)
                os.chmod(d, stat.S_IWRITE)
                # print(d, os.access(d, os.W_OK))


def copyFile(src: str, dst: str) -> str:
    """
    Copies files and removes readonly

    Parameters
    ----------
    src : str
        Source path
    dst : str
        Destination path

    Returns
    -------
    dst_file : str
        Destination file path
    """

    if not os.path.exists(dst):
        os.makedirs(dst)

    dst_file = shutil.copy2(src, dst)
    os.chmod(dst_file, stat.S_IWRITE)
    return dst_file


def copyFileAndReplace(infile: str, outfile: str, replace: list):
    """
    Copy text file and replace content

    Parameters
    ----------
    infile : string
    outfile : string
    replace : list of tuples
    """
    py_src = ''
    with open(infile, 'r') as file:
        for line in file:
            py_src = py_src + line
    for r in replace:
        py_src = py_src.replace(r[0], r[1])

    with open(outfile, 'w+') as file:
        file.write(py_src)


def replaceInFile(file: str, old: str, new: str):
    """Replace text in file

    Parameters
    ----------
    file : str
        File name
    old : str
        Text to be replaced
    new : str
        Text replaced
    """
    with fileinput.FileInput(file, inplace=True) as f:
        for line in f:
            print(line.replace(old, new), end='')
