#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 10197 $
$LastChangedDate:: 2021-05-10 18:04:23 +0200#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""

import os
import sys
import sphinx
from symspace.help.autodoc import AutoDoc
from symspace.osutils.pathutils import Paths


class SyMSpaceHelp:

    def __init__(self):
        symspace_home = Paths.getSyMSpaceRoot()
        self.source_dir = os.path.join(symspace_home, 'Help')
        self.output_dir = os.path.join(symspace_home, 'Help', '_build', 'html')
        self.autodoc_dir = os.path.join(self.source_dir, '_topics', 'DeveloperGuide', 'CodeDocumentation')

        self.build_options = [' ', '-b', 'html', self.source_dir, self.output_dir]

    @staticmethod
    def writeHead():
        print('\n#####################################')
        print('#       SyMSpace Help System          #')
        print('#######################################\n')

    def build(self, exit=False, create_auto_doc=True):
        symspace_home = Paths.getSyMSpaceRoot()
        system_src = os.path.join(symspace_home, 'System', 'Python', 'src')
        general_src = os.path.join(symspace_home, 'System', 'General', 'Python', 'src')

        if create_auto_doc:
            doc = AutoDoc(self.autodoc_dir)
            doc.run(system_src)
            doc.run(general_src)

        self.writeHead()

        val = sphinx.build_main(self.build_options)

        if val == 0:
            print('\nSphinx documentation finished successfully\n\n')
        else:
            print('\nSphinx documentation finished with error, retVal: {}\n\n'.format(val))
        if exit:
            sys.exit(val)


if __name__ == '__main__':
    import argparse
    from symspace.osutils.pathutils import CMDParser

    arg_parser = argparse.ArgumentParser(description='Create SyMSpace Help.')
    arg_parser.add_argument('--autoDoc', dest='autoDoc', action="store_true",
                            help='Automatically create documentation for python packages.')

    args = CMDParser.jepSysArgs()
    args = arg_parser.parse_args(args)
    create_auto_doc = args.autoDoc

    runner = SyMSpaceHelp()
    runner.build(create_auto_doc=create_auto_doc)
