# -*- coding: utf-8 -*-
"""
Created on Thu Okt 06 16:39:02 2017

@author: mkitzberger
"""

import unittest
import numpy as np
from symspace.wrapper.prettyprint import SymPrettyPrintConverter, SymList


class TestClass:
    def __init__(self, val):
        self.val = val


class TestPrettyPrint(unittest.TestCase):

    def test_identity_dict(self):
        d1 = {'a': [1, 2, 3], 'b': [1, 2, 3], 'c': [1, 2, 3]}
        d2 = SymPrettyPrintConverter.ugly2pretty(d1)
        '''Test object identity before and after conversion'''

        self.assertEqual(id(d1['a']), id(d2['a']))

        self.assertEqual(id(d1['b']), id(d2['b']))
        self.assertEqual(id(d1['c']), id(d2['c']))

        '''Remove object and test again the object identity'''
        d2.pop('a')
        self.assertEqual(id(d1['b']), id(d2['b']))
        self.assertEqual(id(d1['c']), id(d2['c']))

        '''Test with recursive conversion'''
        d1 = {'a': [1, {'a': [1, 2, 3], 'b': np.array([1, 2, 3])}]}
        d2 = SymPrettyPrintConverter.ugly2pretty(d1)

        self.assertEqual(id(d1['a']), id(d1['a']))
        self.assertEqual(id(d1['a'][1]['a']), id(d2['a'][1]['a']))

        '''Remove object and test again the object identity'''
        val2 = d2['a'][1]
        val2 = val2.pop('a')

        self.assertTrue(isinstance(val2, SymList))

    @unittest.skip
    def test_identity_list(self):
        self.fail()

    def test_identity_np_array(self):
        phi0 = np.array([1, 2, 3, 4, 5])
        phi1 = SymPrettyPrintConverter.ugly2pretty(phi0)

        obj1 = TestClass(1)
        obj2 = TestClass(2)
        obj3 = TestClass(3)
        arr1 = np.array([obj1, obj2, obj3], dtype=np.object)
        arr2 = SymPrettyPrintConverter.ugly2pretty(arr1)

        '''Check object identity after conversion'''
        self.assertEqual(id(obj1), id(arr2[0]))
        self.assertEqual(id(obj2), id(arr2[1]))
        self.assertEqual(id(obj3), id(arr2[2]))


if __name__ == '__main__':
    unittest.main()
