/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 10085 $
 * $Date: 2020-12-29 09:13:44 +0100 (Di., 29 Dez 2020) $
 */
package at.lcm.symspace.rest;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.nio.channels.Channels;
import java.util.HashMap;
import java.util.UUID;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

import at.lcm.bu21.general.utils.FileUtils;
import at.lcm.bu21.magopt.structure.Component;
import at.lcm.bu21.magopt.structure.IParameter;
import at.lcm.bu21.magopt.structure.ParamContainer;
import at.lcm.bu21.magopt.structure.Parameter;
import at.lcm.bu21.magopt.structure.Project;
import at.lcm.bu21.magopt.utils.MagOptConfig;
import at.lcm.symspace.utils.SmartComponentLocation;
import jakarta.ws.rs.Consumes;
import jakarta.ws.rs.DELETE;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.OPTIONS;
import jakarta.ws.rs.PUT;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.WebApplicationException;
import jakarta.ws.rs.core.MediaType;
import jakarta.ws.rs.core.Response;
import magopt.utils.MopFile;

@Path("projectdetail")
public class ProjectDetails { // implements IProject {

	static private Project project = null;
	static private String webId = null;
	static private String projectId = null;

	public static String getProjectId() {
		return projectId;
	}

	public static Project getProject(String projectId) {
		if (!ProjectDetails.projectId.equals(projectId)) {
			throw new WebApplicationException("Project-identifier not found");
		}

		return project;
	}

	public static void setProject(Project project) {
		ProjectDetails.project = project;
		webId = (String) project.getJSONValue("webId");
		projectId = (String) project.getJSONValue("projectId");

		if (projectId == null) {
			projectId = UUID.randomUUID().toString();
			project.setJSONValue("projectId", projectId);
		}
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	public String getProjectDetails() {
		HashMap<String, Object> map = new HashMap<String, Object>();

		map.put("webId", webId);
		map.put("projectId", ProjectDetails.projectId);
		map.put("projectName", FileUtils.splitName(project.getMopFile().getName())[0]);
		map.put("file", project.getMopFile().getName());
		map.put("path", project.getMopFile().getParent());
		map.put("tree", RESTServer.getBaseUri() + "/tree?projectid=" + ProjectDetails.projectId);

		return JSONObject.toJSONString(map);
	}

	@OPTIONS
	public Response optionsFoListResource() {
		return Response.status(200).header("Allow", "GET").header("Content-Type", MediaType.APPLICATION_JSON)
				.header("Content-Length", "0").build();
	}

	@PUT
	@Consumes(MediaType.APPLICATION_JSON)
	@Path("webid")
	public void setWebId(@QueryParam("projectid") String projectId, @QueryParam("webid") String webId, String str) {

		if (projectId != null && !ProjectDetails.projectId.equals(projectId)) {
			throw new WebApplicationException("Project-identifier not found");
		}

		// TODO remove output
		System.out.println("PUT webid (" + projectId + "):");
		System.out.println("WebId: " + webId);
		System.out.println(str);

		if (projectId != null && webId != null) {
			ProjectDetails.webId = webId;
			project.setJSONValue("webId", webId);
		} else {
			try {
				JSONObject json = (JSONObject) new JSONParser().parse(str);
				if (projectId != null || ProjectDetails.projectId.equals(json.get("projectId"))) {
					webId = (String) json.get("webId");
					project.setJSONValue("webId", webId);
				} else {
					throw new WebApplicationException("Project-identifier not found");
				}
			} catch (ParseException e) {
				throw new WebApplicationException(e);
			}
		}
	}

	@OPTIONS
	@Path("webid")
	public Response optionsFoListResourceWebId() {
		return Response.status(200).header("Allow", "PUT").header("Content-Type", MediaType.APPLICATION_JSON)
				.header("Content-Length", "0").build();
	}

	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("component")
	public String listComponent(@QueryParam("projectid") String projectId) {
		JSONArray comps;
		JSONObject comp;

		if (projectId == null || !ProjectDetails.projectId.equals(projectId)) {
			throw new WebApplicationException("Project-identifier not found");
		}

		// TODO remove output
		System.out.println("GET component (" + projectId + "):");
		comps = new JSONArray();

		for (Parameter p : project.getParameters(ParamContainer.class)) {
			Component c = ((ParamContainer) p).getComponent();
			if (c != null && c.getJSONValue("componentId") != null) {
				comp = new JSONObject();
				comp.put("componentId", c.getJSONValue("componentId"));
				comp.put("component", c.getJSONValue("component"));
				comp.put("domain", c.getJSONValue("domain"));
				comps.add(comp);
			}
		}

		return comps.toJSONString();
	}

	@PUT
	@Consumes(MediaType.APPLICATION_JSON)
	@Path("component")
	public void addComponent(@QueryParam("projectid") String projectId, String str) {
		URL url;
		URLConnection uc;
		InputStream is = null;
		FileOutputStream fos = null;
		MopFile mopc;
		Project proj;
		ParamContainer parent;
		Component comp = null;

		if (projectId == null || !ProjectDetails.projectId.equals(projectId)) {
			throw new WebApplicationException("Project-identifier not found");
		}

		// TODO remove output
		System.out.println("PUT component (" + projectId + "):");
		System.out.println(str);

		try {
			JSONObject json = (JSONObject) new JSONParser().parse(str);

			String componentId = (String) json.get("componentId");
			String domain = (String) json.get("domain");
			String component = (String) json.get("component");
			String token = (String) json.get("token");
			String path = (String) json.get("mopc");
			Object rules = json.get("rules");

			mopc = new MopFile(String.format("%s%s__tmp_rest__%scomponent.mopc", project.getAllFilesDirectory(),
					File.separator, File.separator));
			if (mopc.getParentFile().exists()) {
				FileUtils.delete(mopc.getParentFile());
			}
			mopc.getParentFile().mkdirs();

			url = new URL(String.format("%s/%s", MagOptConfig.getConfig("WebguiURL"), path));
			uc = url.openConnection();
			uc.setRequestProperty("Authorization", token);
			try {
				is = uc.getInputStream();
				fos = new FileOutputStream(mopc);
				fos.getChannel().transferFrom(Channels.newChannel(is), 0, Long.MAX_VALUE);
			} catch (IOException e) {
				System.err.println("Error by reading component-file");
				System.out.println("URL: " + url);
				System.err.println(e.getMessage());
				e.printStackTrace();
				throw new WebApplicationException(e);
			} finally {
				if (is != null) {
					is.close();
				}
				if (fos != null) {
					fos.close();
				}

			}

			proj = Project.load(mopc);
			comp = proj.getComponent();
			if (comp == null) {
				for (IParameter p : proj.getParam()) {
					if (comp == null && p instanceof ParamContainer) {
						comp = ((ParamContainer) p).getComponent();
					}
				}
			}

			comp.setJSONValue("component", component);
			comp.setJSONValue("domain", domain);
			comp.setJSONValue("componentId", componentId);
			project.info("Add component: " + component + "(" + domain + ")");
			parent = SmartComponentLocation.getSmartContainer(project, proj);
			project.importTemplate(proj, parent);

			FileUtils.delete(mopc.getParentFile());
		} catch (Exception e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
			if (is != null) {
				try {
					is.close();
				} catch (IOException e1) {
				}
			}
			if (fos != null) {
				try {
					fos.close();
				} catch (IOException e1) {
				}
			}
			throw new WebApplicationException(e);
		}
	}

	@DELETE
	@Consumes(MediaType.APPLICATION_JSON)
	@Path("component")
	public void rmComponent(@QueryParam("projectid") String projectId, @QueryParam("componentid") String componentId) {
		Component del = null;

		if (projectId == null || !ProjectDetails.projectId.equals(projectId)) {
			throw new WebApplicationException("Project-identifier not found");
		}

		// TODO remove output
		System.out.println("DELETE component (" + projectId + "):");
		System.out.println(componentId);

		for (Parameter p : project.getParameters(ParamContainer.class)) {
			Component c = ((ParamContainer) p).getComponent();
			if (c != null && c.getJSONValue("componentId") != null) {
				if (c.getJSONValue("componentId").equals(componentId)) {
					del = c;
				}
			}
		}

		if (del != null) {
			project.info(
					"Remove component: " + del.getJSONValue("component") + " (" + del.getJSONValue("domain") + ")");
			if (del.getContainer() instanceof Project) {
				Parameter[] params = project.getParam();
				for (int k = params.length - 1; k >= 0; k--) {
					params[k].removeParam();
				}
				del.getContainer().removeComponent();
			} else {
				del.getContainer().removeParam();
			}
		} else {
			throw new WebApplicationException("Component not found");
		}
	}

	@OPTIONS
	@Path("component")
	public Response optionsFoListResourceAddCompoent() {
		return Response.status(200).header("Allow", "GET, PUT, DELETE")
				.header("Content-Type", MediaType.APPLICATION_JSON).header("Content-Length", "0").build();
	}

}
