/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 10128 $
 * $Date: 2021-03-10 11:08:49 +0100 (Wed, 10 Mar 2021) $
 */
package at.lcm.symspace.rest.param;

import at.lcm.bu21.magopt.structure.ParamCollector;
import at.lcm.bu21.magopt.structure.ParamComboBox;
import at.lcm.bu21.magopt.structure.ParamContainer;
import at.lcm.bu21.magopt.structure.ParamDirectory;
import at.lcm.bu21.magopt.structure.ParamField;
import at.lcm.bu21.magopt.structure.ParamFile;
import at.lcm.bu21.magopt.structure.ParamFormula;
import at.lcm.bu21.magopt.structure.ParamFunction;
import at.lcm.bu21.magopt.structure.ParamJavaFunction;
import at.lcm.bu21.magopt.structure.ParamLinkedField;
import at.lcm.bu21.magopt.structure.ParamMaterial;
import at.lcm.bu21.magopt.structure.ParamMatlabFunction;
import at.lcm.bu21.magopt.structure.ParamPythonFunction;
import at.lcm.bu21.magopt.structure.Parameter;
import at.lcm.bu21.magopt.structure.Project;
import at.lcm.symspace.rest.ProjectDetails;
import at.lcm.symspace.rest.RESTServer;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.MediaType;

@Path("parameter")
public class RESTParameter { // implements IParameter {

	protected Parameter getParameter(String projectId, String pid) {
		Project proj;
		Parameter param = null;

		proj = ProjectDetails.getProject(projectId);
		if (param != null) {
			param = proj.getParamByPid(pid);
		}
		return param;
	}

	protected String getObjectUrl(Parameter param) {
		StringBuilder sb = new StringBuilder();

		if (param != null) {
			sb.append(RESTServer.getBaseUri());
			if (param instanceof Project) {
				sb.append("project");
			} else if (param instanceof ParamJavaFunction) {
				sb.append("javafunction");
			} else if (param instanceof ParamPythonFunction) {
				sb.append("pythonfunction");
			} else if (param instanceof ParamMatlabFunction) {
				sb.append("matlabfunction");
			} else if (param instanceof ParamFunction) {
				sb.append("function");
			} else if (param instanceof ParamCollector) {
				sb.append("collector");
			} else if (param instanceof ParamMaterial) {
				sb.append("material");
			} else if (param instanceof ParamContainer) {
				sb.append("container");
			} else if (param instanceof ParamFormula) {
				sb.append("formula");
			} else if (param instanceof ParamLinkedField) {
				sb.append("linkedfield");
			} else if (param instanceof ParamComboBox) {
				sb.append("combobox");
			} else if (param instanceof ParamField) {
				sb.append("field");
			} else if (param instanceof ParamFile) {
				sb.append("file");
			} else if (param instanceof ParamDirectory) {
				sb.append("directory");
			} else {
				sb.append("parameter");
			}
			sb.append("?projectid=");
			sb.append(ProjectDetails.getProjectId());
			sb.append(",pid=");
			sb.append(param.getPId());
		}

		return sb.toString();

	}

	@GET
	@Produces(MediaType.TEXT_PLAIN)
	public String toString2(@QueryParam("project") String project, @QueryParam("pid") String pid) {
//		throw new WebApplicationException(404);
		return getParameter(project, pid).toString();
	}

	@GET
	@Path("/pid")
	@Produces(MediaType.TEXT_PLAIN)
	public String getPId(@QueryParam("project") String project, @QueryParam("pid") String pid) {
		return getParameter(project, pid).getPId();
	}

	@GET
	@Path("/parent")
	@Produces(MediaType.TEXT_PLAIN)
	public String getParent(@QueryParam("project") String project, @QueryParam("pid") String pid) {
		return getObjectUrl(getParameter(project, pid).getParent());
	}

	@GET
	@Path("/identifier")
	@Produces(MediaType.TEXT_PLAIN)
	public String getIdentifier(@QueryParam("project") String project, @QueryParam("pid") String pid) {
		return getParameter(project, pid).getIdentifier();
	}

	@GET
	@Path("/short-identifier")
	@Produces(MediaType.TEXT_PLAIN)
	public String getShortIdentifier(@QueryParam("project") String project, @QueryParam("pid") String pid) {
		return getParameter(project, pid).getShortIdentifier();
	}

	@GET
	@Path("/name")
	@Produces(MediaType.TEXT_PLAIN)
	public String getName(@QueryParam("project") String project, @QueryParam("pid") String pid) {
		return getParameter(project, pid).getName();
	}

	// XXX setName
//	@PUT
//	@Path("/{project}/{id}/name")
//	public void setName(@PathParam("project") String project, @PathParam("pid") String pid, String name) {
//		try {
//			getParam(pid).setName(name);
//		} catch (Exception e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//	}

	@GET
	@Path("/description")
	@Produces(MediaType.TEXT_PLAIN)
	public String getDescription(@QueryParam("project") String project, @QueryParam("pid") String pid) {
		return getParameter(project, pid).getDescription();
	}

	// XXX setDescription

	@GET
	@Path("/comment")
	@Produces(MediaType.TEXT_PLAIN)
	public String getComment(@QueryParam("project") String project, @QueryParam("pid") String pid) {
		return getParameter(project, pid).getComment();
	}

	// XXX setComment

	@GET
	@Path("/index")
	@Produces(MediaType.TEXT_PLAIN)
	public int getIndex(@QueryParam("project") String project, @QueryParam("pid") String pid) {
		return getParameter(project, pid).getIndex();
	}

	// XXX setIndex

	@GET
	@Path("/length")
	@Produces(MediaType.TEXT_PLAIN)
	public int getLength(@QueryParam("project") String project, @QueryParam("pid") String pid) {
		return getParameter(project, pid).getLength();
	}

	@GET
	@Path("/data")
	@Produces(MediaType.TEXT_PLAIN)
	public String getData(@QueryParam("project") String project, @QueryParam("pid") String pid) {
		return getParameter(project, pid).getData().toString();
	}

//	@Override
//	public String getPId() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public String getName() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public void setName(String name) throws RemoteException, Exception {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public String getDescription() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public void setDescription(String description) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public String getComment() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public void setComment(String comment) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public String getIdentifier() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public String getShortIdentifier() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public int getIndex() throws RemoteException {
//		// TODO Auto-generated method stub
//		return 0;
//	}
//
//	@Override
//	public void setIndex(int index) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public int getLength() throws RemoteException {
//		// TODO Auto-generated method stub
//		return 0;
//	}
//
//	@Override
//	public TData getData() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public TData getData(int index) throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public void setData(TData data) throws RemoteException, Exception {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public boolean getPrint() throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public void setPrint(boolean print) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public boolean getConstant() throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public void setConstant(boolean constant) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public boolean getUsed() throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public boolean getReadOnly() throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public boolean getModified() throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public void setModified(boolean modified) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public IParameter[] getUsedBy() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public IParameter[] getSetBy() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public IParameter removeParam() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public void moveUp() throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void moveDown() throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public int getIndexOfParameter() throws RemoteException {
//		// TODO Auto-generated method stub
//		return 0;
//	}
//
//	@Override
//	public boolean getUncertain() throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public boolean getError() throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public boolean[] getAllError() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public void setError(boolean error) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void refresh() throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public IParamContainer getParentParameter() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public IParameter getRoot() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public IParameter getUniqueObject(String uniqueIdentifier) throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public IParameter getObject(String identifier) throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public IParameter getParamByPid(String pid) throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public IParameter[] getParameters(String c) throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public IParameter[] getAnalysisParams() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public IParameter[] getDynamicParams() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public void info(String message) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void info(String message, StackTraceElement[] stack) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void info(String identifier, String message, StackTraceElement[] stack) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void warning(String message) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void warning(String message, StackTraceElement[] stack) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void warning(String identifier, String message, StackTraceElement[] stack) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void error(String message) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void error(String message, StackTraceElement[] stack) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void error(String identifier, String message, StackTraceElement[] stack) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public String getUniqueDocFileName(String file) throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public File getUniqueDocFileName(File file) throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public DocFile[] getDocFiles() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public DocFile getDocFile(String name) throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public File getFileFromDocFile(DocFile doc) throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public void registerDocFile(String name, String strFile, String description)
//			throws RemoteException, NoSuchAlgorithmException, IOException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void unregisterDocFiles() throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void unregisterDocFile(String name) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void resetDocFiles() throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void resetDocFiles(int index) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void deleteDocFiles() throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public void deleteDocFiles(int index) throws RemoteException {
//		// TODO Auto-generated method stub
//
//	}
//
//	@Override
//	public boolean checkDocFiles() throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public boolean checkDocFiles(int index) throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public boolean checkDocFile(String docfile) throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public boolean checkDocFile(int index, String docfile) throws RemoteException {
//		// TODO Auto-generated method stub
//		return false;
//	}
//
//	@Override
//	public String getDsId() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public File getParamDirectory() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public String toStringI() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public List<String> listTabNames() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	@Override
//	public IParameter getClientImpl() throws RemoteException {
//		// TODO Auto-generated method stub
//		return null;
//	}
}
