/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9720 $
 * $Date: 2020-03-19 15:59:49 +0100 (Thu, 19 Mar 2020) $
 */
import java.io.File;
import java.io.IOException;

import at.lcm.bu21.magopt.structure.Project;
import jobmanager.JobDesc;
import jobmanager.JobType;

/**
 * Test Jobmanger with testjob
 * 
 * @author Werner Koppelsttter
 */
public class TestJobManager {

	/**
	 * Run job local
	 * 
	 * @throws IOException
	 */
	public static void testLocal() throws IOException {
		TestJobManager.test(false);
	}

	/**
	 * Run job on cluster
	 * 
	 * @throws IOException
	 */
	public static void testCluster() throws IOException {
		TestJobManager.test(true);
	}

	/**
	 * Run test
	 * 
	 * @param remoteCompute
	 *            Run local or on cluster
	 * @throws IOException
	 */
	public static void test(boolean remoteCompute) throws IOException {
		String jobID;
		boolean outExists = false;

		JobDesc.setVisible(true);
		jobID = JobDesc.createJob("Project", "Dataset", "Parameter", "JobTest", "jobmanager.TestJob");
		JobDesc.addFile(jobID, new File(Project.getInstallPath() + File.separator + "Tools" + File.separator
				+ "JobManager" + File.separator + "Java" + File.separator + "jobmanager.jar"));
		JobDesc.setRequirement(jobID, "JavaVersion >= \"1.8.0_\"");
		JobDesc.setTimeout(jobID, 60);

		if (remoteCompute) {
			JobDesc.submit(jobID, JobType.CLUSTER);
		} else {
			JobDesc.submit(jobID, JobType.LOCAL);
		}

		try {
			while (!JobDesc.isFinished(jobID)) {
				Thread.sleep(1000);
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		for (File f : JobDesc.getFilelist(jobID)) {
			if (f.getName().compareTo("test.txt") == 0) {
				outExists = true;
			}
		}
		System.out.println(JobDesc.getStatus(jobID) + "\t file: " + outExists);
		JobDesc.remove(jobID);

	}
}
