/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9720 $
 * $Date: 2020-03-19 15:59:49 +0100 (Thu, 19 Mar 2020) $
 */
package jobmanager.server;

/**
 * Client for different computation resources
 *
 * @author Werner Koppelsttter
 */
public abstract class Client {

	/** Status-listener of client */
	protected Thread statusListener = null;

	/** Add new to cluster-queue (add in order) */
	protected boolean add = true;

	/**
	 * Reset to add new jobs (new iteration)
	 */
	public void resetAdd() {
		this.add = true;
	}

	/**
	 * Refresh lists (Add remove external jobs)
	 */
	public abstract void refreshList();

	/**
	 * Refresh list (force listener)
	 */
	public void refresh() {
		if (this.statusListener != null) {
			this.statusListener.interrupt();
		}
	}

	/**
	 * Add job to list
	 * 
	 * @param job
	 *            Job to add
	 * @return Adapter for job
	 */
	public abstract JobAdapter addJob(Job job);

	/**
	 * Remove job from list
	 * 
	 * @param job
	 *            Job to remove
	 */
	protected abstract void removeJob(Job job);

}
