/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 10051 $
 * $Date: 2020-11-05 16:14:17 +0100 (Thu, 05 Nov 2020) $
 */
package jobmanager.server;

import java.util.Arrays;

/**
 * Periodic update of status-list
 * 
 * @author Werner Koppelsttter
 */
public class JobStatusListener extends Thread {

	/** Sleep time in milliseconds */
	final static int time = 5000;

	private boolean sleep = false;

	/**
	 * Create threat for periodic update of list
	 */
	public JobStatusListener() {
		this.start();
	}

	@Override
	public void interrupt() {
		if (this.sleep) {
			super.interrupt();
		}
	}

	/**
	 * Periodic update of list
	 */
	@Override
	public void run() {
		Job[] values;
		while (true) {
			try {
				values = Job.joblist.values().toArray(new Job[0]);
				Arrays.sort(values);
				JobManager.resetAdd();
				for (Job item : values) {
					item.update();
				}

				if (JobManager.view != null) {
					JobManager.view.refresh();
				}

				this.sleep = true;
				Thread.sleep(JobStatusListener.time);
			} catch (InterruptedException e) {
			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				this.sleep = false;
			}
		}
	}

}
