/*
 * This file is part of SyMSpace.
 * 
 * Copyright(c)2020,Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it under 
 * the terms of the GNU General Public License, version 2.0 (GPLv2), as published 
 * by the Free Software Foundation.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY 
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A 
 * PARTICULAR PURPOSE. See the GNU General Public License, version 2.0, for more 
 * details.
 * The GPLv2 can be found in the licenses folder as gpl-2.0.txt.
 * 
 * $Revision: 9720 $
 * $Date: 2020-03-19 15:59:49 +0100 (Thu, 19 Mar 2020) $
 */
package jobmanager.server.local;

import java.util.concurrent.ConcurrentSkipListMap;

import at.lcm.bu21.magopt.utils.MagOptConfig;
import jobmanager.server.Client;
import jobmanager.server.Job;

/**
 * Client for local computation
 *
 * @author Werner Koppelsttter
 */
public class LocalClient extends Client {

	/** Maximal number of execution processes */
	private int MAX_RUNNING = Runtime.getRuntime().availableProcessors();

	/** List of job which are current in local execution queue */
	ConcurrentSkipListMap<String, LocalAdapter> locallist = new ConcurrentSkipListMap<String, LocalAdapter>();

	/**
	 * Instace of existing local-client
	 */
	private static LocalClient instance = null;

	/**
	 * Get one instance of client
	 * 
	 * @return Local-client
	 */
	public static synchronized LocalClient getInstance() {
		if (LocalClient.instance == null) {
			LocalClient.instance = new LocalClient();
		}

		return LocalClient.instance;
	}

	/**
	 * Create a client for local computation
	 */
	private LocalClient() {
		LocalAdapter.client = this;

		try {
			this.MAX_RUNNING = Integer
					.parseInt(MagOptConfig.getConfig("MaxLocalJobs"));
		} catch (Exception ex) {
			this.MAX_RUNNING = Runtime.getRuntime().availableProcessors();
		}

		this.statusListener = new LocalStatusListener();
	}

	@Override
	public void refreshList() {
	}

	@Override
	public LocalAdapter addJob(Job job) {
		LocalAdapter adapter = null;

		if (this.add) {
			if (this.locallist.size() < this.MAX_RUNNING) {
				adapter = new LocalAdapter(job);
				this.locallist.put(job.getIdentifier(), adapter);
			} else {
				this.add = false;
			}
		}

		return adapter;
	}

	@Override
	protected void removeJob(Job job) {
		this.locallist.remove(job.getIdentifier());
	}

}
